<?php

namespace BetterStudio\Framework\Core\Integration;

// use core APIs
use BetterStudio\Core\{
	Module
};

use BetterStudio\Component\Integration\{
	Block as BlockIntegration,
	Control as ControlIntegration
};

// use standard APIs
use BetterStudio\Component\Standard\{
	Block as BLockStandard
};


// use controls lists
use BetterStudio\Component\Control;

// use old fashion APIs
use BF_Shortcodes_Manager;

/**
 * Setup better framework core modules.
 *
 * @since 4.0.0
 */
class Setup implements Module\NeedSetup {

	/**
	 * Setup better framework core modules.
	 *
	 * @since 4.0.0
	 * @return bool
	 */
	public static function setup(): bool {

		add_filter( 'better-studio/blocks/integration/list', [ self::class, 'introduce_blocks_integrations' ] );
		add_filter( 'better-studio/controls/integration/list', [ self::class, 'introduce_fields_integrations' ] );

		// introduce old shortcode classes to new block API.
		add_action( 'better-framework/shortcodes/loaded', [ self::class, 'register_blocks' ] );

		// Integrate controls, blocks with page builder support.
		BlockIntegration\Setup::setup();
		ControlIntegration\Setup::setup();

		// Register standard controls
		Control\Setup::setup();
		Elementor\Setup::setup();
		VisualComposer\Setup::setup();

		return true;
	}

	/**
	 * Introduce BF block/shortcode integration classes.
	 *
	 * @hooked better-studio/blocks/integration/list
	 * @since  4.0.0
	 */
	public static function introduce_blocks_integrations( array $integration ): array {

		$integration['bf-shortcodes'] = ShortcodesIntegration::class;

		return $integration;
	}


	/**
	 * Introduce BF controls/fields integration class.
	 *
	 * @hooked better-studio/controls/integration/list
	 * @since  4.0.0
	 */
	public static function introduce_fields_integrations( array $integration ): array {

		$integration['bf-shortcodes']      = ControlsIntegration::class;

		return $integration;
	}

	/**
	 * @since 4.0.0
	 */
	public static function register_blocks(): void {

		foreach ( BF_Shortcodes_Manager::shortcodes_list() as $shortcode_id => $_ ) {

			BLockStandard\BlockStorage::register(
				$shortcode_id,
				new BlockAdapter( $shortcode_id )
			);
		}
	}
}
