<?php

namespace BetterStudio\Framework\Core\Integration\VisualComposer;

// use integration APIs
use BetterStudio\Core\Module\Exception;
use BetterStudio\Component\Integration\{
	Control as ControlIntegration
};

// use standard APIs
use BetterStudio\Component\Standard\{
	Control as ControlStandards
};

// use controls api
use BetterStudio\Component\Control;

class VCControlsIntegration implements ControlIntegration\ControlIntegration {


	public function register( ControlStandards\StandardControl $control ): bool {

		return vc_add_shortcode_param(
			'bs-' . $control->control_type(),
			[ $this, 'render' ]
		);
	}


	/**
	 * @return string
	 */
	public function render( $props, $value ): string {

		if ( ! isset( $props['type'] ) || ! preg_match( '/^bs\-(?P<control_type>.+)$/', $props['type'], $match ) ) {

			return '';
		}

		if ( $control = ControlIntegration\ControlsStorage::factory( $match['control_type'] ) ) {

			return $this->render_control( $control, $props ?? [], $value );
		}

		return '';
	}

	/**
	 * @param ControlStandards\StandardControl $control
	 * @param array                            $props
	 * @param mixed                            $value
	 *
	 * @return string
	 */
	protected function render_control( ControlStandards\StandardControl $control, array $props, $value ): string {

		if ( ! $control instanceof ControlStandards\HaveRenderDynamic ) {

			return '';
		}

		if ( ! isset( $props['input_class'] ) ) {
			$props['input_class'] = '';
		}

		$props['value']       = $value;
		$props['input_name']  = $props['param_name'];
		$props['input_class'] .= ' wpb_vc_param_value';

		if ( ! empty( $props['heading'] ) ) {

			$props['name'] = $props['heading'];
		}

		$section_class = isset( $options['section_class'] ) ? esc_attr( $options['section_class'] ) : '';

		try {

			return sprintf(
				'<div class="bf-section-container vc-input bf-clearfix %s" data-param-type="%s">%s</div>',
				$section_class,
				esc_attr( $control->control_type() ),
				Control\render_control( $control, $props )
			);
		} catch ( Exception $e ) {

			return 'vc control render error: ' . $e->getMessage();
		}
	}

	/**
	 * @return bool
	 */
	public static function is_enable(): bool {

		return VCBlocksIntegration::is_enable() && VCBlocksIntegration::is_edit_form();
	}
}
