<?php

namespace BetterStudio\Component\Integration\Block;

// use standard APIs
use BetterStudio\Component\Standard\{
	Block as BlockStandards
};

// use core APIs
use BetterStudio\Core\{
	Module
};

// use asset-loader APIs
use BetterStudio\Asset\{
	Enqueue
};

class Setup implements Module\NeedSetup {

	use Module\Singleton;

	/**
	 * Setup module.
	 *
	 * @since 1.0.0
	 */
	public static function setup(): bool {

		if ( is_admin() ) { // in admin or ajax requests

			add_action( 'init', [ static::instance(), 'integrate' ] );

		} else {

			add_action( 'template_redirect', [ static::instance(), 'integrate' ] );
		}

		do_action( 'better-studio/blocks/setup' );

		return true;
	}


	/**
	 * @since 1.0.0
	 */
	public function integrate(): void {

		/**
		 * @var string|object $integration
		 */

		foreach ( apply_filters( 'better-studio/blocks/integration/list', [] ) as $integration ) {

			$this->register(
				\is_object( $integration ) ? $integration :
					new $integration(
						Enqueue\EnqueueScript::instance(),
						Enqueue\EnqueueStyle::instance()
					)
			);
		}
	}

	/**
	 * @param BlockStandards\BlockIntegrationInterface $integration
	 *
	 * @since 1.0.0
	 */
	public function register( BlockStandards\BlockIntegrationInterface $integration ): void {

		$registrar = new IntegrationFireUp( $integration );

		$registrar->register( BlockStandards\BlockStorage::blocks() );
	}
}
