<?php declare( strict_types=1 );

namespace BetterStudio\Component\Standard\Block\UnitTest;

use BetterStudio\Asset\Enqueue;

//
use BetterStudio\Component\Standard\{
	Block as BlockStandards
};
use BetterStudio\Component\TestUtil;


class TestBlockAssets extends TestUtil\TestCase {

	/**
	 * @test
	 */
	public function itShouldNotEnqueueJSWhenItsNotAvailable() {

		$sample_block = new TestUtil\Sample\Block\SampleBlockNoScript();

		$instance = new BlockStandards\BlockAssets(
			Enqueue\EnqueueScript::instance(),
			Enqueue\EnqueueStyle::instance()
		);

		$this->assertFalse(
			$instance->enqueue_js( $sample_block )
		);

	}

	/**
	 * @test
	 * @covers BlockAssets::enqueue_js
	 */
	public function itShouldEnqueueBlockJSFileOnce() {

		$script_1 = [
			'id'  => 'style-1',
			'url' => 'https://cdnjs.cloudflare.com/ajax/libs/axios/0.24.0/axios.min.js'
		];

		$script_2 = [
			'id'   => 'style-2',
			'url'  => 'https://cdnjs.cloudflare.com/ajax/libs/three.js/r128/three.min.js',
			'path' => '/path/to/file',
		];

		$sample_block = new TestUtil\Sample\Block\SampleBlock();
		$sample_block->load_scripts( [ $script_1, $script_2 ] );

		$script_mock = $this->mock->enqueue_script_mock();

		$script_mock->expects( $this->exactly( 2 ) )
		            ->method( 'add' )
		            ->withConsecutive(
			            [
				            $this->equalTo( $script_1['id'] ),
				            $this->equalTo( $script_1['url'] ),
				            $this->equalTo( '' )
			            ],
			            [
				            $this->equalTo( $script_2['id'] ),
				            $this->equalTo( $script_2['url'] ),
				            $this->equalTo( $script_2['path'] )
			            ]
		            );

		$script_mock->expects( $this->once() )
		            ->method( 'enqueue' )
		            ->with( $this->equalTo( [ $script_1['id'], $script_2['id'] ] ) );


		$instance = new BlockStandards\BlockAssets(
			$script_mock,
			Enqueue\EnqueueStyle::instance()
		);

		$this->assertTrue(
			$instance->enqueue_js( $sample_block )
		);
	}

	/**
	 * @test
	 */
	public function itShouldNotEnqueueCssWhenItsNotAvailable() {

		$sample_block = new TestUtil\Sample\Block\SampleBlockNoStyle();

		$instance = new BlockStandards\BlockAssets(
			Enqueue\EnqueueScript::instance(),
			Enqueue\EnqueueStyle::instance()
		);

		$this->assertFalse(
			$instance->enqueue_css( $sample_block )
		);

	}

	/**
	 * @test
	 * @covers BlockAssets::enqueue_css
	 */
	public function itShouldEnqueueBlockStyleCssFileOnce() {

		$style_1 = [
			'id'  => 'style-1',
			'url' => 'https://cdnjs.cloudflare.com/ajax/libs/axios/0.24.0/axios.min.js'
		];

		$style_2 = [
			'id'   => 'style-2',
			'url'  => 'https://cdnjs.cloudflare.com/ajax/libs/three.js/r128/three.min.js',
			'path' => '/path/to/file',
		];

		$sample_block = new TestUtil\Sample\Block\SampleBlock();
		$sample_block->load_styles( [ $style_1, $style_2 ] );
		//
		$style_mock = $this->mock->enqueue_style_mock();
		$style_mock->expects( $this->exactly( 2 ) )
		           ->method( 'add' )
		           ->withConsecutive(
			           [
				           $this->equalTo( $style_1['id'] ),
				           $this->equalTo( $style_1['url'] ),
				           $this->equalTo( '' )
			           ],
			           [
				           $this->equalTo( $style_2['id'] ),
				           $this->equalTo( $style_2['url'] ),
				           $this->equalTo( $style_2['path'] )
			           ]
		           );

		$style_mock->expects( $this->once() )
		           ->method( 'enqueue' )
		           ->with( $this->equalTo( [ $style_1['id'], $style_2['id'] ] ) );

		$instance = new BlockStandards\BlockAssets(
			Enqueue\EnqueueScript::instance(),
			$style_mock
		);

		$this->assertTrue(
			$instance->enqueue_css( $sample_block )
		);
	}

}