<?php declare( strict_types=1 );

namespace BetterStudio\Component\Integration\Control\UnitTest;

// use standard APIs
use BetterStudio\Component\Integration\Control as Module;

// use test APIs
use BetterStudio\Component\TestUtil\{
	Sample\Control as ControlSample
};

class TestSetup extends ControlIntegrationTestCase {

	function setUp() {

		parent::setUp();

		Module\ControlsStorage::flush();
	}


	public function testControlsId() {

		Module\ControlsStorage::register( 'control-id', new ControlSample\SampleControl([]) );
		Module\Setup::instance()->integrate();

		$controls = Module\ControlsStorage::controls();

		foreach ( $controls as $control_id => $control ) {

			$this->assertEquals( $control_id, $control->control_type() );
		}
	}

	/**
	 * @test
	 */
	public function itShouldFireRegisterMethodPerControls() {

		$sample_control  = new ControlSample\SampleControl([]);
		$sample_control2 = new ControlSample\SampleControlNoStyle([]);
		$mock            = $this->control_integration_mock();

		$mock->expects( $this->exactly( 2 ) )
		     ->method( 'register' )
		     ->withConsecutive( [ $this->equalTo( $sample_control ) ], [ $this->equalTo( $sample_control2 ) ] );

		Module\ControlsStorage::register( $sample_control->control_type(), $sample_control );
		Module\ControlsStorage::register( $sample_control2->control_type(), $sample_control2 );
		Module\Setup::instance()->integrate();
	}
}
