<?php

namespace BetterStudio\Component\Standard\Control;

use BetterStudio\Asset\Enqueue;

class ControlAssets {

	/**
	 * Store Enqueue JS instance
	 *
	 * @var Enqueue\EnqueueInterface
	 */
	protected $enqueue_js;

	/**
	 * Store Enqueue CSS instance
	 *
	 * @var Enqueue\EnqueueInterface
	 *
	 * @since 1.0.0
	 */
	protected $enqueue_css;

	/**
	 * @param Enqueue\EnqueueInterface $enqueue_js
	 * @param Enqueue\EnqueueInterface $enqueue_css
	 *
	 * @since 1.0.0
	 */
	public function __construct( Enqueue\EnqueueInterface $enqueue_js, Enqueue\EnqueueInterface $enqueue_css ) {

		$this->enqueue_js  = $enqueue_js;
		$this->enqueue_css = $enqueue_css;
	}

	/**
	 * Enqueue the control JS file.
	 *
	 * @param StandardControl $control
	 *
	 * @since 1.0.0
	 * @return bool true on success
	 */
	public function enqueue_js( StandardControl $control ): bool {

		if ( ! $control instanceof HaveScripts ) {

			return false;
		}

		if ( $scripts = $control->scripts_list() ) {

			$handles = $this->add( $scripts, $control->control_type() . '-', $this->enqueue_js );
		}

		return $this->enqueue_js->enqueue( $handles ?? [] );
	}

	/**
	 * Enqueue the control css file.
	 *
	 * @param StandardControl $control
	 *
	 * @since 1.0.0
	 * @return bool true on success
	 */
	public function enqueue_css( StandardControl $control ): bool {

		if ( ! $control instanceof HaveStyles ) {

			return false;
		}

		if ( $styles = $control->styles_list() ) {

			$handles = $this->add( $styles, $control->control_type() . '-', $this->enqueue_css );
		}

		return $this->enqueue_css->enqueue( $handles ?? [] );
	}


	/**
	 * @param array                    $list
	 * @param string                   $handle_prefix
	 * @param Enqueue\EnqueueInterface $enqueue
	 *
	 * @since 1.0.0
	 * @retun string[]
	 */
	protected function add( array $list, string $handle_prefix, Enqueue\EnqueueInterface $enqueue ): array {

		$handles = [];

		foreach ( $list as $item ) {

			if ( ! isset( $item['id'], $item['url'] ) ) {

				continue;
			}

			$handle_id = $handle_prefix . $item['id'];

			$enqueue->add(
				$handle_id,
				$item['url'],
				$item['path'] ?? false,
				$item['deps'] ?? []
			);
			$handles[] = $handle_id;

		}

		return $handles;
	}
}
