<?php

namespace BetterStudio\Component\Standard\Control\UnitTest;

use BetterStudio\Component\Standard\Control\StandardControl;

use BetterStudio\Component\TestUtil\{
	Sample\Control as SampleControl,
	TestCase
};

class TestStandardControl extends TestCase {


	/**
	 * @covers StandardControl::__construct
	 */
	public function testTheConstructor() {

		$options = $this->sampleControlOptions();
		$control = new SampleControl\SampleStandardControl( $options );

		$this->property_get( $control, 'options', $actual );

		$this->assertEquals(
			$options,
			$actual,
			'The Constructor must fill the $options property.'
		);
	}

	/**
	 * @test
	 * @covers StandardControl::__construct
	 */
	public function theConstructorShouldExceptOnlyArrays() {

		$this->expectException( \TypeError::class );
		new SampleControl\SampleStandardControl( 'wrong-value' );
	}

	/**
	 * @test
	 */
	public function theConstructorArgumentIsOptional() {

		$this->expectNotToPerformAssertions();

		try {

			new SampleControl\SampleStandardControl();

		} catch ( \ArgumentCountError $e ) {

			$this->fail( 'The construct have one or more required method(s).' );
		}
	}

	/**
	 * @covers StandardControl::__construct
	 */
	public function testOptionsMethod() {

		$options = $this->sampleControlOptions();
		$control = $this->controlInstance( $options );

		$this->assertEquals(
			$options,
			$control->options()
		);
	}

	/**
	 * @test
	 * @covers StandardControl::__construct
	 */
	public function theOptionsMethodShouldAlwaysReturnArray() {

		$control = $this->controlInstance( null );

		$this->assertSame(
			[],
			$control->options()
		);
	}

	/**
	 * @covers StandardControl::control_label
	 */
	public function testControlLabel() {

		$label   = 'the control label';
		$control = $this->controlInstance( compact( 'label' ) );

		$this->assertEquals(
			$label,
			$control->control_label()
		);
	}


	/**
	 * @test
	 * @covers StandardControl::control_label
	 */
	public function theControlLabelWillReturnNullIfItsNotAvailabe() {

		$control = $this->controlInstance( null );

		$this->assertNull( $control->control_label() );
	}

	/**
	 * @covers StandardControl::load_options
	 */
	public function testLoadOptions() {

		$options = $this->sampleControlOptions();
		$control = new SampleControl\SampleStandardControl( [] );
		$control->load_options( $options );

		$this->property_get( $control, 'options', $actual );
		$this->assertSame( $options, $actual );
	}

	///
	/// internal methods
	///

	protected function controlInstance( $options ) {

		$control = new SampleControl\SampleStandardControl( [] );
		$this->property_set( $control, 'options', $options );

		return $control;
	}

	/**
	 * @since 1.0.0
	 * @return array
	 */
	protected function sampleControlOptions() {

		return [
			'name'        => __( 'Drop Down', 'better-studio' ),
			'desc'        => __( 'Description', 'better-studio' ),
			'id'          => 'select_field_1',
			'type'        => 'select',
			'options'     => [
				'show'       => __( 'Show - Top', 'better-studio' ),
				'bottom'     => __( 'Show - Bottom', 'better-studio' ),
				'top-bottom' => __( 'Show - Top & Bottom', 'better-studio' ),
				'hide'       => __( 'Hide', 'better-studio' ),
			],
			'input_class' => 'input_class'
		];
	}

}
