// @flow
declare var wp: Object;
declare var wpApiSettings: Object;

import {CacheMemory} from './Cache';

const cache = new CacheMemory();

export const config = {

    endPoint: "betterstudio/v1/control-data"
};

export async function request(path: string, data: Object): Promise<Object | boolean> {

    const result = await wp.apiRequest({
        path,
        data,
        method: 'POST',
    }).promise();

    if (!result?.data) {

        throw false;
    }


    return Promise.resolve(result.data);
}

export async function fetch_data(controlType: string, data?: Object): Promise<Object|boolean> {

    return request(config.endPoint, {
        type: controlType,
        params: data
    });
}

export async function load_data(controlType: string, data?: Object, cacheKey: string): Promise<Object|boolean> {

    if (cache.cache_exists(cacheKey)) {

        return cache.cache_get(cacheKey);
    }

    const result = fetch_data(controlType, data)
        .then((response) => {

            cache.cache_set(cacheKey, response);

            return response;
        })
        .catch((error) => {

            cache.cache_delete(cacheKey);

            return error;
        });

    cache.cache_set(cacheKey, result);

    return result;
}

export async function fetch_secure_props(controlType: string, props?: Object): Promise<Object|boolean> {

    return request("betterstudio/v1/control-props", {
        type: controlType,
        props
    });
}

export async function load_secure_props(controlType: string, props?: Object, cacheKey: string): Promise<Object|boolean> {

    if (cache.cache_exists(cacheKey)) {

        return cache.cache_get(cacheKey);
    }

    const result = fetch_secure_props(controlType, props)
        .then((response) => {

            cache.cache_set(cacheKey, response);

            return response;
        })
        .catch((error) => {

            cache.cache_delete(cacheKey);

            return error;
        });

    cache.cache_set(cacheKey, result);

    return result;
}


export class RequestOptions {

    static url(append: string): string {

        const base = wpApiSettings?.root || "";
        return base.replace(/\/$/, "") + "/" + append;
    }

    static nonce(): string {

        return wpApiSettings?.nonce || "";
    }

    static params(): Object {

        return {_wpnonce: this.nonce()};
    }
}