// @flow
import lodash from "lodash"
import {RequestOptions, config} from "./AjaxRequest"

export function parse_url(str: string): Object {

    if (-1 === str.indexOf("?")) {

        return {url: str, params: {}};
    }

    const query = str.split("?"),
        params = query[1].split("&") || [],
        url = query[0];

    const results = {};
    //
    results.url = url;
    results.params = {};


    params.forEach((value: string) => {

        const item = value.split("=");

        results.params[item[0]] = item[1];
    });

    return results;
}

export function redirect(url: string, params: Object, method: "POST" | "GET") {

    const form = document.createElement('form'),
        parse = parse_url(url);
    //
    form.action = parse.url;
    form.method = method.toUpperCase() === "POST" ? "POST" : "GET";
    form.style.display = "none";

    for (const key in lodash.extend(parse.params, params)) {

        const input = document.createElement("input");
        input.type = "hidden";
        input.name = key;
        input.value = parse.params[key];

        form.append(input);
    }

    document.body?.appendChild(form);
    form.submit();

}

export function redirectRest(endpoint: string, params: Object, method: "POST" | "GET") {

    redirect(
        RequestOptions.url(endpoint),
        lodash.extend(params, RequestOptions.params()),
        method
    );
}

export function redirectToControl(controlType: string, params: Object, method: "POST" | "GET") {

    redirectRest(
        config.endPoint,
        {
            type: controlType,
            params: JSON.stringify(params)
        },
        method
    );
}
