const fs = require('fs');
const WPExtractionPlugin = require('@wordpress/dependency-extraction-webpack-plugin');
const NAMESPACE = "@betterstudio";

class DependencyExtraction extends WPExtractionPlugin {

    libraries = [];

    constructor(options) {

        super(options);

        this.options.requestToExternal = this.requestToExternal.bind(this);
    }

    requestToExternal(request) {

        if (request.startsWith(NAMESPACE)) {

            this.libraries.push(request);
        }
    }

    stringify(asset) {

        this.libraries.forEach((lib) => {

            const libPath = require.resolve(lib),
                libAssets = this.readJsonFile(libPath.replace(/\.jsx?$/, '.asset.json'));

            if (libAssets.dependencies) {

                asset.dependencies = asset.dependencies.concat(libAssets.dependencies);
            }
        });


        asset.dependencies = asset.dependencies.concat( Array.isArray(this.options?.requiredScripts) ? this.options.requiredScripts : [] );

        if (asset.dependencies) {

            // filter duplicate items
            asset.dependencies = asset.dependencies.filter((dep, pos) => {

                return asset.dependencies.indexOf(dep) === pos;
            });
        }

        return super.stringify(asset);
    }

    readJsonFile(filePath) {

        try {

            return JSON.parse(fs.readFileSync(filePath)) || {};

        } catch (e) {

            return {};
        }
    }
}

module.exports = DependencyExtraction;