// @flow
import {ControlBase, ControlHaveData, ControlInterface} from "../../ControlBase"
import {ProFeatureModal, option} from '../Features/script'

export default class extends ControlBase implements ControlInterface, ControlHaveData {

    context: any;
    listElement: HTMLElement;
    onClickCallback: Function;

    controlType(): string {

        return 'advance_select';
    }

    init(element: HTMLElement): boolean {

        this.context = element;
        const listElement = this.context.querySelector(".bf-advanced-select-group");

        if (!listElement) {

            return false;
        }
        this.listElement = listElement;
        this.onClickCallback = this.onClick.bind(this);
        this.listElement.addEventListener("click", this.onClickCallback);

        return true;
    }

    onClick(event) {

        if (event.target.tagName === "UL") {

            return;
        }
        const element = event.target.tagName !== 'li' ? event.target.closest('li') : event.target;

        // is item disabled?
        if (element.classList.contains('disable')) {

            return;
        }
        // is pro?
        if (element.classList.contains('pro-feature')) {

            ProFeatureModal(element)

            if (!option(element, 'selectable', false)) {

                return;
            }
        }

        // is none select allowed
        if (this.activeItemsLength() !== 1 || !element.classList.contains('active') || this.allowDeselect()) {

            element.classList.toggle('active');
        }

        if (!this.isMultiple()) {

            this.siblings(element).map(element => element.classList.remove('active'));
        }

        this.save();
    }

    isMultiple(): boolean {

        return this.context.querySelector(".bf-advanced-select").classList.contains('multiple');
    }

    activeItemsLength(): number {

        return this.listElement.querySelectorAll("li.active").length;
    }

    allowDeselect(): boolean {

        return this.context.querySelector(".bf-advanced-select").classList.contains('allow_deselect');
    }

    siblings(element: HTMLElement): Array<HTMLElement> {

        return Array.from(element.parentElement.children).filter((el) => el !== element);
    }

    save() {

        this.valueSet(
            this.collectActiveItems()
        );
    }

    collectActiveItems(): Array<string> {

        return Array.from(this.context.querySelectorAll("li.active")).map((li) => li.dataset.value);
    }

    valueSet(value: mixed): boolean {

        const input = this.context.querySelector('.value');

        if (Array.isArray(value)) {

            value = value.join(',');

        } else if (typeof value !== "string") {

            return false;
        }

        input.value = value;
        input.dispatchEvent(new Event('change', {bubbles: true}));
        this.onChange(input.value);

        return true;
    }

    valueGet(): any {

        return this.context.querySelector('.value').val();
    }

    destroy(): void {

        this.listElement?.removeEventListener("click", this.onClickCallback);
    }

    dataType(): string {

        return 'string';
    }
}

