import classnames from 'classnames'
import Icon from "../../Icon"
import {cssObject} from "../../utils";
import lodash from "lodash"
import {container_attributes} from "../../../js/HtmlHelper";

export default function AdvanceSelect(props) {

    let values = [];

    if (lodash.isArray(props.value)) {

        values = props.value;

    } else if (lodash.isString(props.value)) {

        values = props.value.split(',');

    } else if (!lodash.isUndefined(props.value)) {

        values = [props.value];
    }

    return <div {...container_attributes(props, ['bf-advanced-select', props.status_class])}>
        <ul className="bf-advanced-select-group">
            {
                props.choices.map(([option_id, option]) => {

                    const active = values.indexOf(option_id) !== -1;
                    const disable = option.disable || false;

                    return <li key={option_id} className={classnames(option.classes, {active,disable})}
                               data-value={option_id}
                               style={cssObject(option.inline_styles)}
                    >

                        <Icon icon={option.icon?.icon} className="icon"/>
                        <span className="label" dangerouslySetInnerHTML={{__html: option.label}}/>

                        {!lodash.isEmpty(option.badge) &&
                        <span className={classnames("badge", option.badge?.classes)} style={cssObject(option.badge?.inline_styles)}>
                            <Icon icon={option.badge?.icon?.icon} className="icon"/>
                            {option.badge?.label}
                        </span>}
                    </li>
                })
            }
        </ul>

        <input type="hidden" name={props.input_name} className={classnames(props.input_class, "value")} value={props.value || ""}/>
    </div>
}
