<?php

namespace BetterStudio\Component\Control\AjaxAction;

// use current component
use BetterStudio\Component\Control as LibRoot;

// use standard APIs
use BetterStudio\Component\Standard\{
	Control as ControlStandard
};

class AjaxActionControl extends LibRoot\BaseControl implements
	ControlStandard\HaveStyles,
	ControlStandard\HaveAjaxHandler,
	ControlStandard\WillModifyProps {

	/**
	 * @since 1.0.0
	 * @return string
	 */
	public function control_type(): string {

		return 'ajax_action';
	}

	/**
	 * @since 1.0.0
	 * @return string
	 */
	public function template_dir(): string {

		return __DIR__ . '/templates';
	}

	/**
	 * @inheritDoc
	 */
	public function ajax_handler(): ControlStandard\HandleAjaxRequest {

		return new AjaxActionAjaxHandler();
	}

	public function modify_props( array $props ): array {

		if ( ! empty( $props['callback'] ) ) {

			$props['token'] = wp_create_nonce( sprintf( 'bf-custom-callback:%s', $props['callback'] ?? '' ) );
		}

		if ( ! isset( $props['button-class'] ) ) {

			$props['button-class'] = '';
		}

		if ( ! isset( $props['primary'] ) || $props['primary'] ) {

			$props['button-class'] .= ' button-primary';
		}

		return $props;
	}
	
	public function styles_list(): array {

		return [
			[
				'id' => 'buttons',
			]
		];
	}
}
