// @flow
import {ControlBase, ControlInterface, ControlHaveData} from "../../ControlBase"

export default class extends ControlBase implements ControlInterface, ControlHaveData {

    context: HTMLElement;

    constructor(props: ?ControlProps) {

        super(props);

        this.refreshValues = this.refreshValues.bind(this);
    }


    controlType(): string {

        return 'checkbox';
    }

    init(element: HTMLElement): boolean {

        this.context = element;

        this.bindEvents();

        return true;
    }

    bindEvents() {

        this.context.addEventListener('change', this.refreshValues);
    }

    checkboxes() {

        return this.context.querySelectorAll('[type="checkbox"]');
    }

    refreshValues() {

        const values = {};

        this.checkboxes().forEach((checkbox: HTMLInputElement) => {

            if (!checkbox.checked) {

                return;
            }

            const key = checkbox.dataset.key;

            if (key) {
                values[key] = checkbox.value;
            }
        });

        this.onChange(values);
    }

    valueSet(value: mixed): boolean {

        return true;
    }

    valueGet(): any {

        return [];
    }

    destroy(): void {

        this.context?.removeEventListener('change', this.refreshValues);
    }

    dataType(): string {

        return 'object';
    }
}
