// @flow
import {ControlBase, ControlInterface, ControlHaveData} from "../../ControlBase"

export default class extends ControlBase implements ControlInterface, ControlHaveData {

    context: HTMLElement;

    constructor(props: ?ControlProps) {

        super(props);

        this.onTextareaChanged = this.onTextareaChanged.bind(this);
    }

    controlType(): string {

        return 'code';
    }

    init(element: HTMLElement): boolean {

        this.context = element;

        this.bindEvents();

        return true;
    }

    bindEvents() {

        const textarea = this.textarea();

        textarea && textarea.addEventListener("change", this.onTextareaChanged);
    }

    onTextareaChanged(event) {

        this.onChange(event.target.value);
    }

    textarea(): ?HTMLTextAreaElement {

        return this.context.querySelector('.bf-code-editor');
    }

    valueSet(value: mixed): boolean {

        const textarea = this.textarea();

        if (!textarea) {
            return false;
        }

        textarea.value = value;

        return true;
    }

    valueGet(): any {

        const textarea = this.textarea();

        return textarea && textarea.value;
    }

    destroy(): void {

        const textarea = this.textarea();

        textarea && textarea.removeEventListener("change", this.onTextareaChanged);
    }

    dataType(): string {

        return 'string';
    }
}
