<?php

namespace BetterStudio\Component\Control\Core\Rest;

// use core modules
use BetterStudio\Core\{
	Module\Exception
};

// use standard APIs
use BetterStudio\Component\Standard\{
	Control as ControlStandard
};

// use lib functions
use function BetterStudio\Component\Control\{
	json_decode
};

// use wp core APIs
use WP_HTTP_Response;

class RestRequestData extends RestRequestBase {

	/**
	 * @throws Exception
	 *
	 * @return array
	 */
	public function response(): ?WP_HTTP_Response {

		$control = $this->control();
		$params  = $this->request->get_param( 'params' );

		if ( \is_string( $params ) ) {

			$params = json_decode( $params );
		}

		return $control->ajax_handler()->handle_request(
			\is_array( $params ) ? $params : []
		);
	}


	/**
	 * @throws Exception
	 *
	 * @since 1.0.0
	 * @return ControlStandard\HaveAjaxHandler
	 */
	protected function control(): ControlStandard\HaveAjaxHandler {
		if ( ! $control_type = $this->request->get_param( 'type' ) ) {

			throw new Exception( 'invalid request', 'rest-data-invalid-request' );
		}

		if ( ! $control = Utils::control( $control_type ) ) {

			throw new Exception( 'invalid control type given.', 'invalid-control' );
		}

		if ( ! $control instanceof ControlStandard\HaveAjaxHandler ) {

			throw new Exception( 'invalid control given.', 'control-error' );
		}

		return $control;
	}

	/**
	 * @since 1.0.0
	 * @return string
	 */
	public function rest_end_point(): string {

		return 'control-data';
	}

	/**
	 * The validation rules.
	 *
	 * @since 1.0.0
	 * @return string[]
	 */
	public function validation_rules(): array {

		return [
			'type' => 'required|bs-control: handle_ajax_request=1',
		];
	}
}