<?php

namespace BetterStudio\Component\Control\Core\Rest;

// use core modules
use BetterStudio\Core\{
	Module\Exception
};

// use standard APIs
use BetterStudio\Component\Standard\{
	Control as ControlStandard
};

// use wp core APIs
use WP_HTTP_Response;

class RestRequestProps extends RestRequestBase {

	/**
	 * @throws Exception
	 *
	 * @return WP_HTTP_Response|null
	 */
	public function response(): ?WP_HTTP_Response {

		$control = $this->control();
		$props   = $this->props();
		$token   = $this->token();

		if ( empty( $token ) || $control->secure_props_token( $props ) !== $token ) {

			throw new Exception( 'invalid token', 'invalid-token' );
		}

		$new_props = $control->secure_props( $props );

		if ( $control instanceof ControlStandard\WillModifyProps ) {

			$new_props = $control->modify_props( $new_props );
		}

		unset( $new_props['_lazy_loading'] );

		return new WP_HTTP_Response( [
			'data'  => $new_props,
			'extra' => $this->extra_params(),
		] );
	}


	protected function extra_params(): array {

		return array_diff_key(
			$this->request->get_params(), [
			'props' => '',
			'token' => '',
			'type'  => '',
		] );
	}

	protected function token(): string {

		if ( ! $token = $this->request->get_param( 'token' ) ) {

			$props = $this->request->get_param( 'props' );
			$token = $props['_token'] ?? $props['token'] ?? '';
		}

		return trim( $token );
	}

	protected function props(): array {

		$props = $this->request->get_param( 'props' );

		unset( $props['_token'] );

		return \is_array( $props ) ? $props : [];
	}

	/**
	 * @throws Exception
	 *
	 * @since 1.0.0
	 * @return ControlStandard\HaveSecureProps
	 */
	protected function control(): ControlStandard\HaveSecureProps {

		if ( ! $control_type = $this->request->get_param( 'type' ) ) {

			throw new Exception( 'invalid request', 'rest-props-invalid-request' );
		}

		if ( ! $control = Utils::control( $control_type ) ) {

			throw new Exception( 'invalid control type given.', 'invalid-control' );
		}

		if ( ! $control instanceof ControlStandard\HaveSecureProps ) {

			throw new Exception( 'invalid control given.', 'control-error' );
		}

		return $control;
	}

	/**
	 * @since 1.0.0
	 * @return string
	 */
	public function rest_end_point(): string {

		return 'control-props';
	}

	/**
	 * The validation rules.
	 *
	 * @since 1.0.0
	 * @return string[]
	 */
	public function validation_rules(): array {

		return [
			'type' => 'required|bs-control: modify_settings=1',
		];
	}
}