// @flow
import {ControlBase, ControlInterface, ControlHaveData} from "../../ControlBase"
import {fetch_data} from "../../js/AjaxRequest"
import UI from '../../js/UI'
import hooks from "../../js/Hooks";
import lodash from "lodash"

export default class extends ControlBase implements ControlInterface, ControlHaveData {

    context: ?HTMLElement;

    controlType(): string {

        return 'custom';
    }

    init(context: HTMLElement): boolean {

        this.context = context;

        this.deferRender();

        return true;
    }

    deferRender(): boolean {

        const deferElement = this.deferElement();

        if (!deferElement) {

            return false;
        }

        UI.block(this.context);


        fetch_data("custom", lodash.extend(deferElement.dataset, {_value: this.valueGet()}))
            .then((response) => {

                try {

                    UI.unblock(this.context);

                    if (response) {

                        deferElement.innerHTML = response?.raw || "";

                        hooks.do_action('custom/deferred/loaded', this.deferElement());
                    }

                } catch (error) {
                    console.error(error);
                }

            }).catch(() => {
            UI.unblock(this.context);

            console.error("Error fetch custom html", deferElement);
        });

        return true;
    }

    deferElement(): ?HTMLElement {

        if (!this.context) {

            return null;
        }

        return this.context.querySelector(".bf-custom-field-view");
    }

    valueSet(value: mixed): boolean {

        const input = this.valueInput();

        if(! input) {
            
            return false;
        }

        input.value = value;
        
        return true;
    }

    valueGet(): any {

        const input = this.valueInput();

        return input && input.value;
    }

    valueInput(): ?HTMLInputElement {

        return this.context.querySelector("input.bf-custom-field-values");
    }

    destroy(): void {

    }

    dataType(): string {

        return 'string';
    }
}
