import $ from 'jquery'
import lodash from 'lodash'
import {autop} from "@wordpress/autop";

function getConfig(id) {

    const element = document.getElementById(id);

    if (!element) {

        return;
    }

    if ("application/json" === element.type) {
        return JSON.parse(element.innerHTML);
    }

    return element.innerHTML;
}

function getModalID(element) {

    return element.dataset?.modalId;
}

function extractYoutubeVideoID(video_url) {

    if (lodash.isEmpty(video_url)) {
        return;
    }

    const match = video_url.toString().match(/\:\/\/w*\.?youtube\.com\/*.*?v=([^&]+)/);

    return match && match[1];
}

function templateParams(element) {

    const defaultConfig = getConfig(`cnf-bs-pro-feature-${getModalID(element)}`) || {};

    const templateObject = lodash.extend(
        defaultConfig.template || {}, // modal config
        JSON.parse(element.dataset?.template || '{}')  // override items
    );

    templateObject.video_id = extractYoutubeVideoID(templateObject.video_url);

    return lodash.mapValues(templateObject, (string, key) => {

        const result = string ? Mustache.render(string, element.dataset || {}) : '';

        if (key.match(/desc/i)) {

            return autop(result);
        }

        return result;
    })
}

export function options(element, _default) {

    if (!element?.dataset?.options) {

        return {};
    }

    try {

        return JSON.parse(element.dataset.options);

    } catch (e) {

        return {};
    }
}

export function option(element, option_key, _default) {

    const allOptions = options(element);

    return allOptions[option_key] || _default;
}

export function ProFeature(context) {

    $(context).on('click', ".bs-pro-feature", function (event) {

        event.stopPropagation();

        const element = event.target.classList.contains('bs-pro-feature') ? event.target : event.target.closest('.bs-pro-feature');

        ProFeatureModal(element);
    });
}

var _init = false;

export function ProFeatureModal(element) {

    if (!_init) {

        $.bs_modal_template('bs-pro-feature', getConfig('tmpl-bs-pro-feature'));
        $.bs_modal_template('bs-pro-feature-after', getConfig('tmpl-bs-pro-feature-after'));

        _init = true;
    }

    const onModalInserted = function (modal) {

        // init
        const $modalWrapper = modal.$modal.parent();
        const canvas = document.createElement('canvas');
        //
        canvas.classList.add('interaction');
        $modalWrapper[0] && $modalWrapper[0].appendChild(canvas);

        const modalIconUnlock = () => {
            $modalWrapper.find('.bs-pro-feature-modal').toggleClass('bs-modal-state-lock bs-modal-state-unlock');
        };

        const modalIconLock = () => {
            $modalWrapper.find('.bs-pro-feature-modal').toggleClass('bs-modal-state-lock bs-modal-state-unlock');
        };

        const onButtonHovered = (event) => {
            modalIconUnlock();
        };

        const onButtonMouseOut = () => {
            modalIconLock();
        };

        const onButtonClicked = () => {
            modal.change_skin({
                template: 'bs-pro-feature-after'
            });
        };

        modal.$modal.on('mouseover', '.button-primary', onButtonHovered);
        modal.$modal.on('click', '.button-primary', onButtonClicked);
        modal.$modal.on('mouseout', '.button-primary', onButtonMouseOut);
    };

    $.bs_modal({
        close_button: true,
        modalId: "bs-pro-feature-modal",
        modalClass: "bf-fields-style",
        styles: {
            container: 'max-width:555px'
        },
        template: 'bs-pro-feature',
        content: templateParams(element),
        events: {
            after_append_html: onModalInserted
        }
    });
}
