// @flow
import {ControlBase, ControlInterface} from "../../ControlBase"
import lodash from 'lodash'
import $ from 'jquery'
import ui from '../../js/UI'
import {RequestOptions, config} from '../../js/AjaxRequest'

export default class extends ControlBase implements ControlInterface {

    context: any;
    data: Object;

    constructor(props: ?ControlProps) {

        super(props);

        this.onClick = this.onClick.bind(this);
    }

    controlType(): string {

        return 'import';
    }

    init(element: HTMLElement): boolean {

        this.context = element;

        this.context.querySelector(".import-upload-btn")?.addEventListener("click", this.onClick);

        $('.import-file-input').fileupload({
            limitMultiFileUploads: 1,
            url: RequestOptions.url(config.endPoint),
            autoUpload: false,
            replaceFileInput: false,
            formData: this.formData(),
            add: (event, data) => {
                this.data = data;
            },
            start: function () {
                ui.panel_loader('loading');
            },
            done:  (e, data)=> {

                const result = data.result || {};

                if (result.success) {

                    ui.panel_loader('succeed', result.message || "");

                } else {

                    ui.panel_loader('error', result.message || "");
                }

                if (result.refresh) {

                    this.reload_location(1500);
                }

            },
            error: function () {

                ui.panel_loader('error');
            },
            drop: function () {
                return false;
            }
        });

        return true;
    }

    reload_location(delay: number): void {

        ui.panel_loader('hide');

        setTimeout(function () {
            window.location.reload();
        }, delay || 0);
    }

    formData(): Object {


        return lodash.extend(RequestOptions.params(), {
            type: this.controlType(),
            params: this.elementParams(),
        })
    }

    elementParams(): Object {

        const element = this.context.querySelector("input[type=file]");

        return JSON.stringify(lodash.extend({}, element?.dataset));
    }

    onClick() {

        this.data && this.data.submit();
    }

    destroy(): void {

        this.context.querySelector("button")?.removeEventListener("click", this.onClick);
    }
}
