// @flow
import $ from 'jquery'
import {ControlBase, ControlInterface, ControlHaveData} from "../../ControlBase"
import lodash from "lodash";
import {autop} from "@wordpress/autop";

export default class extends ControlBase implements ControlInterface {

    controlType(): string {

        return 'modal_connector';
    }

    init(element: HTMLElement): boolean {

        if ('undefined' !== typeof element) {

            let addItem = element.querySelector('.bs-mc-insert-item');
            addItem && addItem.addEventListener('click', event => {
                openModalConnector(event, element);
            });
        }

        return true;
    }

    destroy(): void {

    }
}

function templateParams(event, element) {

    let connectorClass = '.bs-modal-connector';
    let targetWrapper = event.target.closest(connectorClass);

    if (!targetWrapper || !targetWrapper.getAttribute('data-modal-type')) {

        return {};
    }

    const defaultConfig = getConfig(`bs-modal-connector-${targetWrapper.getAttribute('data-modal-type')}`) || {};

    const templateObject = lodash.extend(
        defaultConfig.template || {}, // modal config
        JSON.parse(element.dataset?.template || '{}')  // override items
    );

    templateObject.title = targetWrapper.getAttribute('data-name');

    let wrapper = document.createElement('div');
    let bodyFields = JSON.parse(targetWrapper.getAttribute('data-body-fields'));
    let labels = Object.values(bodyFields);

    bodyFields && Object.keys(bodyFields).forEach((field, index) => {

        let innerWrapper = document.createElement('div');
        innerWrapper.classList.add('bs-modal-connector-control');
        innerWrapper.innerHTML = `<label for="${field}">${labels[index]}</label><input type="text" name="${field}" value="">`;

        wrapper.innerHTML += innerWrapper.outerHTML;
    });

    templateObject.body = wrapper.outerHTML;
    templateObject.jsCallback = targetWrapper.getAttribute('data-js-callback');
    templateObject.serviceID = targetWrapper.getAttribute('data-service');
    templateObject.connector = connectorClass;

    return lodash.mapValues(templateObject, (string, key) => {

        const result = string ? Mustache.render(string, element.dataset || {}) : '';

        if (key.match(/body/i)) {

            return autop(result);
        }

        return result;
    });
}

function getConfig(id) {

    const element = document.getElementById(id);

    if (!element) {

        return;
    }

    if ("application/json" === element.type) {
        return JSON.parse(element.innerHTML);
    }

    return element.innerHTML;
}

function getModalID(element) {

    return element.dataset?.modalId;
}

var _init = false;

function openModalConnector(event, element) {

    event.preventDefault();

    let target = event.target;

    if ('A' !== target.tagName) {

        target = target.closest('.bs-mc-insert-item');
    }

    if (!_init) {

        $.bs_modal_template('bs-modal-connector', getConfig('tmpl-bs-modal-connector'));

        _init = true;
    }

    $.bs_modal({
        close_button: true,
        modalId: "bs-modal-connector-modal",
        modalClass: "bf-fields-style",
        styles: {
            container: 'max-width:555px'
        },
        template: 'bs-modal-connector',
        content: templateParams(event, element),
        events: {
            after_append_html: function () {
                handleConfirm(this,target);
            },
        },
    });
}

function handleConfirm(modalObject,secondTarget) {

    modalObject.$modal.find('a.button-primary').on('click', event => {

        event.preventDefault();

        let callback = event.target.getAttribute('data-callback');

        if ('undefined' === typeof GlobalAppConnector) {

            return false;
        }

        let glob = new GlobalAppConnector('bf');

        if ('undefined' === typeof glob[callback]) {

            return false;
        }

        glob[callback](event.target,secondTarget);

        let closeButton = modalObject.$modal.find('.bs-close-modal');
        closeButton && closeButton.click();
    });
}