// @flow
import {ControlBase, ControlInterface, ControlHaveData} from "../../ControlBase"

export default class extends ControlBase implements ControlInterface, ControlHaveData {

    context: HTMLElement;

    constructor(props: ?ControlProps) {

        super(props);

        this.onRadioChanged = this.onRadioChanged.bind(this);
    }


    controlType(): string {

        return 'radio';
    }

    init(element: HTMLElement): boolean {

        this.context = element;

        this.bindEvents();

        return true;
    }

    bindEvents() {

        this.context.addEventListener('change', this.onRadioChanged);
    }

    valueSet(value: mixed): boolean {

        this.context.querySelectorAll('input[type="radio"]').forEach((element) => {

            element.checked = element.value === value;

        });
        return true;
    }

    valueGet(): any {

        const radio = this.context.querySelector("input:checked");

        return radio && String(radio.value);
    }

    destroy(): void {

        this.context?.removeEventListener('change', this.onRadioChanged);
    }

    onRadioChanged() {

        this.onChange(this.valueGet());
    }

    dataType(): string {

        return 'string';
    }
}
