import $ from 'jquery'
import hooks from "../../js/Hooks";

export default function (context, onChange) {

    var $context = $(context);

    // Add jQuery UI Sortable to Repeater Items
    $context.find('.bf-repeater-items-container').sortable({
        revert: true,
        cursor: 'move',
        delay: 150,
        handle: ".bf-repeater-item-title",
        start: function (event, ui) {
            ui.item.addClass('drag-start');
        },
        beforeStop: function (event, ui) {
            ui.item.removeClass('drag-start');
        }
    });

    // Remove Repeater Item
    $context.on('click', '.bf-remove-repeater-item-btn', function (e) {

        var $this = $(this);

        if ($this.hasClass('no-event')) {

            return;
        }

        var $section = $this.closest('.bf-section');

        if ($(".bf-repeater-items-container>.bf-repeater-item", $section).length === 1) {

            alert("Can not remove last item!");

            return;
        }

        if (confirm('Are you sure?')) {

            /**
             * Append hidden input before remove last item
             */
            if ($('.bf-repeater-item:visible', $section).length === 1) {
                var inputName = $(':input:first', $section).attr('name').toString().match(/([^\[]+)/)[0]

                var $placeholder = $('<input>', {
                    'class': 'placeholder-input',
                    'name': inputName + '[]',
                    'type': 'hidden'
                });
                $section.append($placeholder);
            }

            $(this).closest('.bf-repeater-item').slideUp(function () {

                $(this).remove();

                hooks.do_action('repeater/item/removed', $section[0]);
            });
        }

        e.preventDefault();
    });

    // Collapse
    $context.on('click', '.handle-repeater-item', function () {
        $(this).toggleClass('bf-closed').closest('.bf-repeater-item').find('.repeater-item-container').slideToggle(400);
    });

    // Clone Repeater Item by click
    $context.on('click', '.bf-clone-repeater-item', function (e) {
        e.preventDefault();

        var $this = $(this);

        if ($this.hasClass('no-event')) {

            return;
        }

        var $repeater_items_container = $this.siblings('.bf-repeater-items-container'),
            _html = $context.find('.repeater-item-tmpl').html(),
            count = $repeater_items_container.find('>*').size();

        // Retrieve script tags

        if (!_html) {
            return;
        }

        _html = _html.replace(/{{iteration}}/g, count);

        $repeater_items_container.append(
            _html
        );

        // Event for when new item added
        $this.closest('.bf-section').trigger('repeater_item_added', [$repeater_items_container]);

        hooks.do_action('repeater/item/added', $repeater_items_container.find('.bf-repeater-item:last')[0]);

        // Remove temporary placeholder input
        $repeater_items_container.closest('.bf-section').find('input.placeholder-input').remove();
    });

    return {$context};
}
