import {Component} from "@wordpress/element";
import lodash from 'lodash'
import {__} from "@wordpress/i18n"
import {container_attributes} from "../../../js/HtmlHelper";

export default class extends Component {

    constructor(props) {

        super(props);

        let items = [];

        if (props.value && Array.isArray(props.value)) {
            items = props.value;
        }

        this.state = {items};
    }

    appendItem() {

        const defaults = this.defaults(),
            items = this.items();

        const item = lodash.clone(defaults[0]);
        item.uniqueKey = this.generateKey();

        items.push(item);

        this.setState({items});
    }

    defaults() {

        if (Array.isArray(this.props['default'])) {

            return this.props['default'];
        }

        const items = {};

        Object.entries(this.props?.options || {}).forEach(([key, control]) => {

            if (control.id) {

                items[control.id] = "";
            }
        })

        return [items];
    }

    itemChanged(value) {

        const items = this.repeater.state.items || {};

        if (!items[this.i]) {
            items[this.i] = {};
        }

        items[this.i] = lodash.clone(items[this.i]);
        items[this.i][this.id] = value;

        const newItems = lodash.clone(items);

        this.repeater.setState({items: lodash.cloneDeep(newItems)});
        this.repeater.onChange(newItems);
    }

    onChange(items) {

        this.props.onChange && this.props.onChange(items || this.items());
    }

    prepareChildrenElements(elements, elementsValue, i) {

        return elements.map((element) => {

            const id = element.props.id,
                value = elementsValue && elementsValue[id] ? elementsValue[id] : '';

            const uniqueID = id + "__" + i;

            const props = {
                value,
                key: uniqueID,
                id: uniqueID,
                input_name: `${id}[${i}]`,
                i,
                onChange: this.itemChanged.bind({repeater: this, id, i})
            };

            if (element.props.children) {

                props.children = this.prepareChildrenElements(Array.isArray(element.props.children) ? element.props.children : [element.props.children], elementsValue, i);
            }

            return React.cloneElement(element, props);
        });
    }

    items() {

        const items = this.state.items || [],
            propsValue = Array.isArray(this.props.value) ? this.props.value : [];

        return items.map((value, key) => {

            return Object.assign({
                uniqueKey: this.generateKey()
            }, value || {}, propsValue[key] || {});

        }).filter((item) => !lodash.isEmpty(item));
    }

    removeItem(removeIndex) {

        if (lodash.size(this.state.items) === 1) {

            alert(__('Can not remove last item!', 'better-studio'));

            return;
        }

        const items = lodash.cloneDeep(
            (this.state.items || {}).filter((item, currentIndex) => currentIndex !== removeIndex)
        );

        this.setState({items});
        this.onChange(items);
    }

    generateKey() {

        return 'item-' + lodash.random(9, 9e6);
    }

    render() {

        return (<div {...container_attributes(this.props, "bs-control-repeater")}>
                <div className="bf-repeater-items-container bf-clearfix">
                    {this.items().map((values, i) => {

                        return (
                            <div className="bf-repeater-item" key={values.uniqueKey}>
                                <div className="bf-repeater-item-title ui-sortable-handle">
                                    <h5>
                                        <span
                                            className="handle-repeater-title-label">{this.props.item_title || __('Item', 'better-studio')}</span>
                                        <span className="handle-repeater-item"> </span>
                                        <span className="bf-remove-repeater-item-btn no-event" onClick={() => this.removeItem(i)}>
                                                {this.props.delete_label || __('Delete', 'better-studio')}
                                            </span>
                                    </h5>
                                </div>

                                <div className="repeater-item-container bf-clearfix">
                                    {this.prepareChildrenElements(Array.isArray(this.props.children) ? this.props.children : [this.props.children], values, i)}
                                </div>
                            </div>
                        )
                    })}
                </div>
                <button className="bf-clone-repeater-item button button-primary bf-main-button no-event"
                        onClick={this.appendItem.bind(this)}
                        dangerouslySetInnerHTML={{__html: this.props.add_label || __('Add', 'better-studio')}}>
                </button>
            </div>
        )
    }
}
