// @flow
import {ControlBase, ControlInterface, ControlHaveData} from "../../ControlBase"

export default class extends ControlBase implements ControlInterface, ControlHaveData {

    context: HTMLElement;

    constructor(props: ?ControlProps) {

        super(props);

        this.onInputChanged = this.onInputChanged.bind(this);
    }

    controlType(): string {

        return 'select';
    }

    init(element: HTMLElement): boolean {

        this.context = element;

        this.bindEvents();

        return true;
    }

    select(): ?HTMLSelectElement {

        return this.context.querySelector("select");
    }

    bindEvents() {

        const select = this.select();

        select && select.addEventListener('change', this.onInputChanged)
    }

    onInputChanged() {

        const values = this.valueGet();

        this.valueSet(values);
    }

    valueSet(value: mixed): boolean {

        this.onChange(value);

        return false;
    }

    valueGet(): Array<string>  {

        const input = this.select();

        if (!input) {

            return [];
        }

        if (!input.multiple) {

            return [input.value];
        }

        const values = [];

        Array.from(input.selectedOptions).forEach((option) => {

            values.push(option.value);
        });

        return values;
    }

    dataType(): string {

        return 'array';
    }

    destroy(): void {

        const input = this.select();

        input && input.removeEventListener('change', this.onInputChanged)
    }
}