import {Component} from "@wordpress/element";
import _ from 'lodash'
import {container_attributes} from "../../../js/HtmlHelper";

export default class extends Component {

    render() {

        const props = this.props,
            elements = [];

        let values = props.multiple ? props.value.toString().split(',') : props.value;

        if (!props.multiple && Array.isArray(values)) {

            values = values.shift();
        }

        if (props.options?.category_walker?.raw) {

            elements.push(<select key="select-raw" name={props.input_name || props.id} className={props.input_class}
                                  multiple={props.multiple} defaultValue={values}
                                  dangerouslySetInnerHTML={{__html: props.options?.category_walker?.raw}}>
            </select>);
        } else {

            elements.push(<select key="select-options" name={props.input_name || props.id} className={props.input_class}
                                  multiple={props.multiple}
                                  defaultValue={values}>{this.optionElements(props.options)}</select>);
        }

        return <div {...container_attributes(props, ["bf-select-option-container", {multiple: !!props.multiple}])}>
            {
                elements
            }
        </div>
    }

    optionElements(options) {

        if (!options) {

            return [];
        }

        const items = [];


        for (const _key in options) {

            if (_.isObject(options[_key])) {
                continue;
            }

            items.push(<option value={_key} key={_key}>{options[_key]}</option>);
        }

        for (const key in options) {

            if (!_.isObject(options[key])) {

                continue;
            }

            const option = _.isObject(options[key]) ? options[key] : {};

            if (option.options) {

                items.push(<optgroup label={option.label} key={key}>
                    {this.optionElements(option.options)}
                </optgroup>);
            } else if (option.label) {

                items.push(<option value={key} key={key} disabled={option.disabled || false}>{option.label}</option>);

            } else if (typeof option !== 'object') {
                
                items.push(<option value={key} key={key} disabled={option.disabled || false}>{option}</option>);
            }
        }

        return items;
    }
}
