import $ from 'jquery'
import { __ } from '@wordpress/i18n';
import {ProFeatureModal, option} from '../Features/script'

export default function (context, onChange) {

    var $context = $(context);

    function getData() {

        var data;

        try {

            data = JSON.parse($context.find('.select-popup-data').text());

        } catch(e) {

            return false;
        }

        if (typeof data === "undefined") {
            return false;
        }



        var obj,
            heading = $context.find('.select-popup-field').data('heading');
        var modal_loc = {
            header: '%%name%%',
            search: __('Search...', 'better-studio'),
            btn_label: __('Choose', 'better-studio'),
            btn_label_active: __('Current', 'better-studio'),
            filter_cat_title: __('Category', 'better-studio'),
            categories: [],
            filter_type_title: __('Type', 'better-studio'),
            all_l10n: __('All', 'better-studio'),
            types: [],
        };

        if (data.texts) {
            [
                ['modal_button', 'btn_label'],
                ['modal_current', 'btn_label_active'],
                ['modal_title', 'header']
            ].forEach(function (replacement) {
                var optKey = replacement[0],
                    modalKey = replacement[1];

                if (data.texts[optKey]) {
                    modal_loc[modalKey] = data.texts[optKey];
                }
            });
        }
        if (modal_loc.header.indexOf('%%name%%') !== -1) {
            modal_loc.header = modal_loc.header.replace('%%name%%', heading);
        }

        var modal2_loc = {
            header: __('Do you want to change %%name%%?', 'better-studio'),
            button_ok: __('Yes, Change', 'better-studio'),
            button_cancel: __('Cancel', 'better-studio'),
            caption: '%s',
        };

        if (data.confirm_texts) {
            modal2_loc = $.extend(modal2_loc, data.confirm_texts);
        }
        if (modal2_loc.header.indexOf('%%name%%') !== -1) {
            modal2_loc.header = modal2_loc.header.replace('%%name%%', heading);
        }
        if (data.column_class) {
            modal_loc.content_class = data.column_class;
        }

        var items = [],
            cats = {},
            types = {};

        var opts = data.options;
        for (var id in opts) {
            obj = opts[id];
            obj.id = id;

            if (typeof obj.cat !== "undefined") {

                obj.cat.forEach(function (cat) {
                    cats[cat] = modal_loc.categories[cat] || cat;
                });
            }

            if (typeof obj.type !== "undefined") {

                obj.type.forEach(function (type) {
                    types[type] = modal_loc.types[type] || type;
                });
            }

            var badges = [];
            if (obj.badges) {
                obj.badges.forEach(function (badge) {
                    var badgeObj = {};
                    badgeObj.badge = badge;

                    badges.push(badgeObj);
                });
            }
            obj.badges = badges;

            items.push(obj);
        }

        //  [ Modal data, General data, Main Modal Localization, Confirm Modal Localization ]
        return [
            {
                items: items,
                cats: cats,
                types: types,
            },
            data,
            modal_loc,
            modal2_loc
        ];
    }


    $context.on('click', '.select-popup-field', function (e) {

        e.preventDefault();

        var initialZIndex = 1.3e5,
            $select = $(this),
            activeItem = $select.find('input.select-value').val(),
            data = getData.call(this);

        // FIX nested bs-modal z-index issue
        var $parentModal = $select.closest('.bs-modal');

        if ($parentModal.length) {
            initialZIndex = parseInt($parentModal.css('z-index'));
        }

        if (data[0]) {

            $.bs_selector_modal({
                bsModal: {
                    destroyHtml: true,
                    show: true
                },

                id: 'better-select-popup-modal',
                modalClass: 'pds-modal',

                itemInnerHtml: '<div class="bf-item-container">\n    \n    <figure>\n        <img src="{{img}}"\n             alt="{{label}}"\n             class="bs-style-thumbnail" data-current-image="{{current_img}}">\n         <div class="bf-item-badges">\n\n         {{#badges}}\n        <div class="bf-item-badge">\n            {{badge}}\n        </div>\n         {{/badges}}\n         </div>\n    </figure>\n\n    <footer class="bf-item-footer bf-clearfix">\n        <span class="bf-item-title">\n            {{label}}\n        </span>\n\n        <div class="bf-item-buttons">\n            <span class="bf-toggle-item-status">\n                <a href="#" target="_blank"\n                   class="button bf-btn-secondary bf-btn-dark">{{btn_label}}</a>\n            </span>\n        </div>\n    </footer>\n</div>',

                content: data[2],

                items: data[0].items,

                categories: data[0].cats,
                types: data[0].types,
                itemsGroupSize: 9,

                fuse: {
                    keys: ['label']
                },

                events: {
                    scrollIntoView: function (elements) {
                        var modal = this;

                        elements.forEach(function (el) {
                            modal.$(el).find('.bf-item-container').addClass('bs-animate bs-fadeInUp');
                        });
                    },

                    after_append_html: function () {
                        var modal = this;

                        function getBody() {
                            var out = '';

                            if (data[3].list_items) {

                                out += '<div class="pdsm-notice-list"><ul class="styled">';

                                data[3].list_items.forEach(function (lbl) {
                                    out += '<li>' + lbl + '</li>';
                                });

                                out += '</ul></div>';
                            }

                            if (data[3].notice) {

                                out += '<div class="pdsm-notice">';
                                out += data[3].notice;
                                out += '</div>';
                            }

                            return out;
                        }

                        function setItemAsActive($item) {

                            $item.addClass('active');
                            var $btn = $(".bf-toggle-item-status a", $item);

                            $btn.attr('class', 'bf-btn-primary disabled').html(data[2].btn_label_active);
                            $btn.parent().removeClass('bf-toggle-item-status');
                            $btn.on('click', function (e) {
                                e.preventDefault();
                            });
                        }

                        function setValue(id, label) {

                            var $input = $select.find('input.select-value');
                            $input.val(id).change();
                            $select.find('.active-item-label').html(label);

                            $input[0].dispatchEvent(new Event('change', {bubbles: true}));

                            onChange(id);
                        }

                        function setImage(src) {
                            $select.find('.select-popup-selected-image img').attr('src', src);
                        }

                        setItemAsActive(modal.selectItem(activeItem));

                        modal.bsModal.$modal.on('click', '.bssm-item:not(.disabled)', function (e) {
                            e.preventDefault();

                            var $this = $(this);

                            if ($this.hasClass('active')) {
                                modal.bsModal.close_modal();

                                return;
                            }

                            if ($this.hasClass('pro-feature')) {

                                ProFeatureModal($this[0])

                                if (!option($this[0], 'selectable', false)) {

                                    return;
                                }
                                return;
                            }

                            var $selectedItem = $this.closest('.bssm-item'),
                                selectedItemId = $selectedItem.data('item-id'),
                                $selectedItemImg = $selectedItem.find('figure img'),
                                selectedItemTitle = $selectedItem.find('.bf-item-title').text();

                            var selectedItemImg = $selectedItemImg.data('current-image') ||
                                $selectedItemImg.attr('src');

                            if (!data[1].confirm_changes) {

                                var e = $.Event('select-popup-select');
                                $select.trigger(e, [this, modal.bsModal]);

                                if (!e.isDefaultPrevented()) {
                                    modal.bsModal.close_modal();
                                    setValue(selectedItemId, selectedItemTitle);
                                    setImage(selectedItemImg);
                                }
                                $select.trigger('select-popup-selectd', [this, modal.bsModal])

                                return;
                            }

                            $.bs_modal({
                                modalId: 'better-select-popup-confirm-modal',
                                modalClass: 'pds-confirm-modal',
                                content: $.extend({
                                    image_align: $('body').hasClass('rtl') ? 'left' : 'right',
                                    image_style: 'margin-left:10px;width:240px',
                                    image_src: $selectedItem.find('.bs-style-thumbnail').attr('src'),
                                    image_caption: data[3].caption.replace('%s', $selectedItem.find('.bf-item-title').text()),

                                    body: getBody()
                                }, data[3]),

                                buttons: {
                                    confirm: {
                                        label: data[3].button_ok,
                                        type: 'primary',
                                        clicked: function () {

                                            setValue(selectedItemId, selectedItemTitle);
                                            setImage(selectedItemImg);

                                            var e = $.Event('select-popup-confirm');
                                            $select.trigger(e, [this, modal.bsModal]);

                                            if (e.isDefaultPrevented()) {
                                                return;
                                            }

                                            var delay = Math.floor(this.options.animations.delay / 2);
                                            this.close_modal();
                                            setTimeout(function () {
                                                modal.bsModal.close_modal();
                                            }, delay);
                                        }
                                    },
                                    close_modal: {
                                        btn_classes: 'bs-modal-button-aside',
                                        label: data[3].button_cancel,
                                        action: 'close',
                                        type: 'secondary',
                                        focus: true
                                    }
                                },

                                template: 'single_image',

                                styles: {
                                    container: 'width: 615px;max-width:100%'
                                },
                                animations: {
                                    body: 'bs-animate bs-fadeInLeft'
                                }
                            });

                        }).on('click', '.bf-toggle-item-status a.disabled', function () {
                            return false;
                        });

                        $select.trigger('select-popup-loaded', [data, modal]);
                    }
                }

            }, {
                initialZIndex: initialZIndex
            });
        }
    });

    return {$context};
}