<?php

namespace BetterStudio\Component\Control;

// use Integration library
use BetterStudio\Component\Integration\{
	Control as ControlIntegration
};

// use utility libraries
use BetterStudio\Utils\{
	Validator
};

// use core libraries
use BetterStudio\Core\{
	Module,
	Rest as RestLib
};

class Setup implements Module\NeedSetup {

	/**
	 * Store list of the registered render callbacks.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	protected static $wrappers = [];

	public static function setup(): bool {

		self::introduce();
		//
		RestLib\RestSetup::register( Core\Rest\RestRequestProps::class );
		RestLib\RestSetup::register( Core\Rest\RestRequestData::class );
		RestLib\RestSetup::setup();
		//
		Validator\Factory::register( 'bs-control', Core\Validation\ControlValidation::class );

		Features\ProFeature::setup();

		return true;
	}

	/**
	 * Register controls list.
	 *
	 * @since 1.0.0
	 */
	public static function introduce(): void {

		ControlIntegration\ControlsStorage::register( 'hr', Hr\HrControl::class );
		ControlIntegration\ControlsStorage::register( 'button', Button\ButtonControl::class );
		ControlIntegration\ControlsStorage::register( 'text', Text\TextControl::class );
		ControlIntegration\ControlsStorage::register( 'info', Info\InfoControl::class );
		ControlIntegration\ControlsStorage::register( 'heading', Heading\HeadingControl::class );
		ControlIntegration\ControlsStorage::register( 'code', Code\CodeControl::class );
		ControlIntegration\ControlsStorage::register( 'radio', Radio\RadioControl::class );
		ControlIntegration\ControlsStorage::register( 'color', Color\ColorControl::class );
		ControlIntegration\ControlsStorage::register( 'media', Media\MediaControl::class );
		ControlIntegration\ControlsStorage::register( 'wp_editor', WpEditorControl::class );
		ControlIntegration\ControlsStorage::register( 'select', Select\SelectControl::class );
		ControlIntegration\ControlsStorage::register( 'editor', Editor\EditorControl::class );
		ControlIntegration\ControlsStorage::register( 'custom', Custom\CustomControl::class );
		ControlIntegration\ControlsStorage::register( 'export', Export\ExportControl::class );
		ControlIntegration\ControlsStorage::register( 'import', Import\ImportControl::class );
		ControlIntegration\ControlsStorage::register( 'slider', Slider\SliderControl::class );
		ControlIntegration\ControlsStorage::register( 'repeater', Repeater\RepeaterControl  ::class );
		ControlIntegration\ControlsStorage::register( 'sorter', Sorter\SorterControl  ::class );
		ControlIntegration\ControlsStorage::register( 'textarea', Textarea\TextareaControl::class );
		ControlIntegration\ControlsStorage::register( 'checkbox', Checkbox\CheckboxControl::class );
		ControlIntegration\ControlsStorage::register( 'switch', SwitchControl\SwitchControl::class );
		ControlIntegration\ControlsStorage::register( 'typography', Typography\TypographyControl::class );
		ControlIntegration\ControlsStorage::register( 'ajax_select', AjaxSelect\AjaxSelectControl::class );
		ControlIntegration\ControlsStorage::register( 'icon_select', IconSelect\IconSelectControl::class );
		ControlIntegration\ControlsStorage::register( 'term_select', TermSelect\TermSelectControl::class );
		ControlIntegration\ControlsStorage::register( 'ajax_action', AjaxAction\AjaxActionControl::class );
		ControlIntegration\ControlsStorage::register( 'media_image', MediaImage\MediaImageControl::class );
		ControlIntegration\ControlsStorage::register( 'image_radio', ImageRadio\ImageRadioControl  ::class );
		ControlIntegration\ControlsStorage::register( 'select_popup', SelectPopup\SelectPopupControl::class );
		ControlIntegration\ControlsStorage::register( 'image_upload', ImageUpload\ImageUploadControl::class );
		ControlIntegration\ControlsStorage::register( 'image_select', ImageSelect\ImageSelectControl  ::class );
		ControlIntegration\ControlsStorage::register( 'image_preview', ImagePreview\ImagePreviewControl::class );
		ControlIntegration\ControlsStorage::register( 'advance_select', AdvanceSelect\AdvanceSelectControl::class );
		ControlIntegration\ControlsStorage::register( 'modal_connector', ModalConnector\ModalConnectorControl::class );
		ControlIntegration\ControlsStorage::register( 'multiple_controls', MultipleControls\MultipleControls::class );
		ControlIntegration\ControlsStorage::register( 'sorter_checkbox', SorterCheckbox\SorterCheckboxControl::class );
		ControlIntegration\ControlsStorage::register( 'background_image', BackgroundImage\BackgroundImageControl::class );
	}


	/**
	 * Register a render callback.
	 *
	 * @param string   $wrapper_id       a unique id for the wrapper .
	 * @param callable $wrapper_callback the render callback.
	 *
	 * @since 1.0.0
	 * @return bool true on success.
	 */
	public static function register_wrapper( string $wrapper_id, callable $wrapper_callback ): bool {

		if ( isset( static::$wrappers[ $wrapper_id ] ) ) {

			return false;
		}

		if ( ! \is_callable( $wrapper_callback ) ) {
			return false;
		}

		static::$wrappers[ $wrapper_id ] = $wrapper_callback;

		return true;
	}

	/**
	 * @param string $wrapper_id
	 *
	 * @return callable|null
	 */
	public static function wrapper( string $wrapper_id ): ?callable {

		return static::$wrappers[ $wrapper_id ] ?? null;
	}
}
