<?php

namespace BetterStudio\Component\Control\Sorter;

use BetterStudio\Component\Control as LibRoot;

// use standard APIs
use BetterStudio\Component\Standard\{
	Control as ControlStandard
};

// use lib functions
use function BetterStudio\Component\Control\{
	json_decode
};

class SorterControl extends LibRoot\BaseDataControl implements
	ControlStandard\WillModifySaveValue,
	ControlStandard\WillModifyProps,
	ControlStandard\HaveSecureProps,
	ControlStandard\HaveScripts {

	/**
	 * @since 1.0.0
	 * @return string
	 */
	public function control_type(): string {

		return 'sorter';
	}

	/**
	 * @since 1.0.0
	 * @return string
	 */
	public function template_dir(): string {

		return __DIR__ . '/templates';
	}

	/**
	 * @param array $props
	 *
	 * @return array
	 */
	public function secure_props( array $props ): array {

		if ( ! empty( $props['deferred-options'] ) ) {

			$props['options'] = LibRoot\Helper::deferred_options( $props['deferred-options'] );
		}

		return $props;
	}

	/**
	 * @param array $props
	 *
	 * @since 1.0.0
	 * @return array
	 */
	public function modify_props( array $props ): array {

		/// Make props value backward compatible
		$props['value']   = $this->modify_save_value( $props['value'] ?? [], $props );
		$props['choices'] = [];

		foreach ( $props['options'] ?? [] as $key => $value ) {

			$props['choices'][] = [ $key, $value ];
		}

		return $props;
	}


	public function scripts_list(): array {

		return [
			[
				'id' => 'jquery-ui-sortable',
			]
		];
	}

	public function secure_props_needed( array $props, bool $use_dynamic_props ): bool {

		return ! empty( $props['deferred-options'] );
	}

	public function secure_props_token( array $props ): string {

		if ( empty( $props['deferred-options'] ) ) {

			return '';
		}

		return LibRoot\Helper::deferred_options_token( $props['deferred-options'] );
	}

	public function modify_save_value( $value, array $props = [] ) {

		if ( \is_string( $value ) ) {

			$value = json_decode( $value );
		}

		if ( ! \is_array( $value ) ) {

			return [];
		}

		$new_struct = [];

		foreach ( $value as $item_id => $item_option ) {

			if ( \is_array( $item_option ) ) {

				if ( ! isset( $item_option['id'] ) ) {

					$item_option['id'] = $item_id;
				}

				$new_struct[] = $item_option;

			} else {

				$new_struct[] = [
					'id'    => $item_id,
					'label' => $item_option,
				];
			}
		}

		return $new_struct;
	}

	public function data_type(): string {

		return 'array';
	}
}
