import {Component} from "@wordpress/element";
import classnames from 'classnames'
import _ from 'lodash'
import {cssObject, findChoice} from "../../utils";
import {container_attributes} from "../../../js/HtmlHelper";

export default class extends Component {

    render() {

        const props = this.props;
        const values = Array.isArray(props.value) ? props.value : [];

        return (
            <div {...container_attributes(props, "bf-sorter-groups-container")}>

                <input name={props.input_name}
                       value={JSON.stringify(values)}
                       className={props.input_class} type="hidden"/>

                <ul id={"bf-sorter-group-" + props.id}
                    className={classnames("bf-sorter-list", "bf-sorter-" + props.id)}>
                    {
                        values.map((item) => {

                            const ID = item.id || lodash.kebabCase(item?.label || item),
                                option = findChoice(props.choices || [], ID) || item,
                                extraStyles = cssObject(option.css || ''),
                                label = _.isObject(option) ? option.label : option,
                                extraClass = option['css-class'] || '';

                            return (
                                <li id={'bf-sorter-group-item-' + props.id + '-' + ID}
                                    className={classnames("item-" + props.id, extraClass)}
                                    style={extraStyles} key={ID} data-id={ID}
                                >
                                    {label}
                                </li>
                            )
                        })
                    }

                    {
                        values.length === 0 && (props.choices || []).map(([key, item]) => {

                            const extraStyles = cssObject(item.css || ''),
                                label = _.isObject(item) ? item.label : item,
                                ID = item.id || key,
                                extraClass = item['css-class'] || '';

                            return (
                                <li id={'bf-sorter-group-item-' + props.id + '-' + ID}
                                    className={classnames("item-" + props.id, extraClass)}
                                    style={extraStyles} key={ID} data-id={ID}
                                >
                                    {label}
                                </li>
                            )
                        })
                    }
                </ul>
            </div>
        )
    }
}
