// @flow
import {ControlBase, ControlInterface, ControlHaveData, HaveDeprecation} from "../../ControlBase"
import $ from 'jquery'

export default class extends ControlBase implements ControlInterface, ControlHaveData, HaveDeprecation {

    impl: any;

    context: HTMLElement;


    constructor(props: ?ControlProps) {

        super(props);

        this.onValueChange = this.onValueChange.bind(this);
    }

    controlType(): string {

        return 'sorter_checkbox';
    }

    init(element: HTMLElement): boolean {

        this.context = element;

        $(".bf-sorter-list", element).sortable({
            placeholder: "placeholder-item",
            cancel: "li.disable-item",
            update: (event) => {

                const $this = $(event.target);

                if (typeof $this.attr('checked') != "undefined") {
                    $this.closest('li').addClass('checked-item');
                } else {
                    $this.closest('li').removeClass('checked-item');
                }

                this.refresh();
            }
        });

        $(":checkbox", element).on('change', this.refresh.bind(this));

        this.bindEvents();

        return true;
    }

    onValueChange() {

        const values = this.valueGet();

        this.onChange(values);
    }

    valueSet(value: mixed): boolean {

        var input = this.context.querySelector("input[type=hidden]");
        input.value = JSON.stringify(value);
        input.dispatchEvent(new Event('change', {bubbles: true}));

        this.onChange(value);

        return true;
    }

    refresh() {

        this.valueSet(this.valueGet());
    }

    valueGet(): Array<Object> {

        const values = [];

        if (!this.context) {

            return values;
        }

        this.context.querySelectorAll('input[type="checkbox"]').forEach((input: HTMLElement) => {

            // input.parentElement.classList[input?.checked ? "remove": "add"]("disable-item");

            if (!input.disabled) {

                values.push({
                    label: input.parentElement?.innerText?.trim(),
                    id: input.dataset.id,
                    active: input.checked,
                })
            }
        });

        return values;
    }

    bindEvents() {

        const listEl = this.context.querySelector(".bf-sorter-list");

        listEl && listEl.addEventListener('change', this.onValueChange);
    }

    destroy(): void {

        const listEl = this.context.querySelector(".bf-sorter-list");

        listEl && listEl.removeEventListener('change', this.onValueChange);
    }

    dataType(): string {

        return 'array';
    }

    static transformData(): Array<Object> {

        return [
            {
                type: "object",

                transformer: (prevData) => {

                    const newData = [];

                    for (const id in prevData) {

                        if (prevData[id] && prevData[id] !== "0") {

                            newData.push({
                                label: id,
                                id,
                                active: true
                            });
                        }
                    }

                    return newData;
                }
            }
        ];
    }
}

