import {Component} from "@wordpress/element";
import {cssObject, findChoice} from "../../utils"
import classnames from 'classnames'
import lodash from 'lodash'
import {container_attributes} from "../../../js/HtmlHelper";

export default class extends Component {

    render() {

        const props = this.props;
        const options = props.choices || [];

        const valuesStatus = {};

        const values = Array.isArray(props.value) ? props.value.map((item, index) => {

            if (!item.id) {

                item.id = lodash.kebabCase(item?.label || index);
            }

            valuesStatus[item.id] = Boolean(item.active);

            return item;
        }) : [];

        const li = (item) => {

            if (lodash.isEmpty(item)) {


                return null;
            }

            const extraStyles = cssObject(item.css || ''),
                extraClass = item['css-class'] || '',
                isActive = item.active || valuesStatus[item.id],
                disabled = item['css-class'] ? item['css-class'].indexOf('active-item') === -1 : false;

            return (
                <li id={'bf-sorter-group-item-' + props.id + '-' + item.id}
                    className={classnames("item-" + props.id, extraClass, {
                        "checked-item": isActive
                    })}
                    style={extraStyles} key={item.id}
                >
                    <input value={item.label} type="checkbox" name={item.id}
                           defaultChecked={isActive}
                           data-id={item.id}
                           disabled={disabled}
                           className="sorter-checkbox-value"/>

                    <span dangerouslySetInnerHTML={{__html: item.label}}></span>
                </li>
            )
        };

        const displayedItems = [];
        const activeAndSelectedItems = values.map((vItem) => {

            if (!vItem.active || typeof vItem.id === 'undefined') {

                return false;
            }
            const item = findChoice(vItem.id);

            if (!item) {

                return false;
            }

            if (item['css-class']?.indexOf('active-item') === -1) {

                return false;
            }

            displayedItems.push(item.id);

            return item;
        }).filter(item => !!item);


        const disabledItems = (options).map(([, item]) => {

            if (displayedItems.indexOf(item.id) !== -1) {

                return false;
            }

            if (item['css-class']?.indexOf('active-item') === -1) {

                return false;
            }

            item.active = typeof valuesStatus[item.id] === 'boolean' ? valuesStatus[item.id] : true;

            displayedItems.push(item.id);

            return item;

        }).filter(item => !!item);

        const deActiveItems = (options).map(([, item]) => {

            return displayedItems.indexOf(item.id) === -1 ? item : false;
        }).filter(item => !!item);

        return (
            <div {...container_attributes(props, "bf-sorter-groups-container")}>

                <input type="hidden" value={props.value} name={props.input_name}
                       className={classnames("bf-term-select-value", props.input_class)}/>

                <ul id={"bf-sorter-group-" + props.id}
                    className={classnames("bf-sorter-list", "bf-sorter-" + props.id)}>
                    {activeAndSelectedItems.map(li)}
                    {disabledItems.map(li)}
                    {deActiveItems.map(li)}
                </ul>
            </div>
        )
    }
}
