import $ from 'jquery'
import _ from 'lodash'

import {__} from '@wordpress/i18n';
import {load_data} from "../../js/AjaxRequest";
import UI from "../../js/UI";

export default function (context, onChange) {

    var config = {
        autoCheckParent: false // Check parent term if all children was checked
    };

    var $context = $(context);

    function getTermIdByInputName(inputName) {
        return inputName.toString().match(/\[(\d+)\]$/i)[1];
    }

    /**
     * Display Primary link only when more than one root term are active
     */
    function canCreatePrimaryLink($container) {
        var $checked = $container.parent()
            .find('.bf-field-term-select-wrapper>ul>li>.bf-checkbox-multi-state[data-current-state="active"]');

        return $checked.length >= 2;
    }

    function toggleShowPrimaryClass($el, addClass) {

        $el[addClass ? 'addClass' : 'removeClass']
        ('bf-field-term-show-primary-label');
    }

    function createPrimaryLink($labelContainer, termID) {

        if (!termID) {
            var $input = $labelContainer.prevAll('.bf-checkbox-multi-state')
                .find('.bf-checkbox-status');
            if (!$labelContainer.length) {
                return false;
            }
            termID = getTermIdByInputName($input.attr('name'));
        }

        if (!termID) {
            return false;
        }

        var $primaryEL = $labelContainer.find(".bf-make-term-primary");
        if (!$primaryEL.length || $primaryEL.attr('class') !== 'bf-make-term-primary') {

            if ($primaryEL.length) {
                $primaryEL.remove();
            }

            var $el = $('<em/>', {
                'class': 'bf-make-term-primary'
            }).html('<a href="#" data-term-id="' + termID + '">' + __('Make Primary', 'better-studio') + '</a>');

            $el.appendTo($labelContainer);

            return true;
        }

        return false;
    }

    function initCheckboxes($context) {

        $('.bf-checkbox-multi-state', $context).each(function () {
            var $this = $(this),
                state = $this.data('current-state'),
                $currentMark = $('[data-state="' + state + '"]', $this);

            $currentMark.css('display', 'inline-block');
            $('[data-state]:not(.disabled)', $this).not($currentMark).css('display', 'none');
        });

        $('.bf-checkbox-multi-state', $context).each(function () {
            var $this = $(this),
                state = $this.data('current-state');

            $('[data-state=' + state + ']', $this).css('display', 'inline-block');

            /**
             * Initial 'Collect ID' Flag
             */
            var $childrenUL = $this.nextAll('.children');

            if ($childrenUL.length) {
                if (isAllChildrenActive($childrenUL)) {
                    updateCollectIdFlag(
                        $childrenUL,
                        'dont-collect'
                    );

                    $childrenUL.addClass('bf-checkbox-dual-state');
                    updateCheckboxStatesList(
                        $childrenUL,
                        ['active', 'deactivate']
                    );
                }
            }
        });
    }

    function updateCheckboxStatesList($context, validStatus) {
        $('[data-state]', $context).each(function () {
            var $this = $(this),
                state = $this.data('state'),
                isEnable = $.inArray(state, validStatus) > -1;

            $this[isEnable ? 'removeClass' : 'addClass']('disabled');
        })
    }

    function changeCheckboxState(status, $checkboxEL, $context) {
        $context = $context || $checkboxEL.parent();

        $checkboxEL.attr('data-current-state', status)
            .data('current-state', status)
            .find('.bf-checkbox-status')
            .val(status);

        $checkboxEL.trigger('bf-checkbox-change', []);
        initCheckboxes($context);

        return true;
    }

    function changeChildrenState($childUlWrapper, status) {

        if (!status) {
            return false;
        }

        if ($childUlWrapper.length) {
            return changeCheckboxState(
                status,
                $childUlWrapper.find('.bf-checkbox-multi-state'),
                $childUlWrapper
            );
        }

        return false;
    }

    function updateCollectIdFlag($context, flag) {
        $('.bf-checkbox-multi-state', $context)
            [flag == 'collect' ? "removeClass" : 'addClass']('bf-checkbox-skip-collect-active-term-id');
    }

    function canCollectTermAsDeactivated(checkboxStatusEL) {
        return checkboxStatusEL.parentElement.dataset.currentState === 'deactivate';
    }

    function canCollectTermAsActivated(checkboxStatusEL, $checkboxContainer) {

        return checkboxStatusEL.parentElement.dataset.currentState === 'active' && (function ($container) {
            return !$container.hasClass('bf-checkbox-skip-collect-active-term-id');
        })($checkboxContainer || $(checkboxStatusEL).closest('.bf-checkbox-multi-state'));
    }

    function isTermPrimary($labelContainer) {
        return $labelContainer.find('.bf-make-term-primary')
            .hasClass('bf-is-term-primary');
    }

    function isAllChildrenActive($childrenUL) {
        return $childrenUL.length && !$('.bf-checkbox-multi-state[data-current-state!="active"]', $childrenUL).length;
    }

    function scrollToFirstActiveCheckbox($context) {
        var topMargin = 40,
            $catsContainer = $('.bf-field-term-select-wrapper', $context);

        if ($catsContainer.length) {
            var $firstChecked = $('.bf-checkbox-multi-state[data-current-state="active"]:first', $catsContainer);

            if ($firstChecked.length) {
                var firstCheckPos = $firstChecked.offset().top - $catsContainer.offset().top - topMargin;
                $catsContainer.animate({scrollTop: "+=" + firstCheckPos}, 300);
            }
        }
    }

    function findPrimaryTermId(selected_items) {

        if (!_.isString(selected_items)) {

            return;
        }

        for (var term_id of selected_items.split(',')) {

            if (term_id.startsWith('+')) {

                return term_id.substr(1);
            }
        }
    }

    function makePrimary(term_id) {

        $context.find(`.cat-item-${term_id} .bf-make-term-primary a`).click();
    }

    $context.on('bf-checkbox-change.term_select', ".bf-checkbox-multi-state", _.debounce(function (e, state, calledFrom) {

        var $this = $(this),
            $container = $this.closest('.bf-field-term-select-wrapper'),
            termsIdList = [];

        var $clickedCheckBox = $this.closest('.bf-checkbox-multi-state'),
            $childrenUL = $clickedCheckBox.nextAll('.children'),
            childStateChanged = calledFrom === 'auto-check-children' || changeChildrenState($childrenUL, state === 'deactivate' ? 'none' : state);

        /**
         * Check parent term if all children terms was checked
         */
        if (config.autoCheckParent && state === 'active' && calledFrom !== 'auto-check-children') {

            var $parentChildren = $clickedCheckBox.closest('ul.children');

            if (isAllChildrenActive($parentChildren)) {

                var $EL = $parentChildren.prevAll('.bf-checkbox-multi-state');
                changeCheckboxState('active', $EL);
                $EL.trigger('bf-checkbox-change', [state, 'auto-check-children']);

                return;
            }
        }

        if (childStateChanged && state === 'active') {
            $childrenUL.addClass('bf-checkbox-dual-state');
            $childrenUL.find('ul.children').addClass('bf-checkbox-dual-state');

            updateCheckboxStatesList(
                $childrenUL,
                ['active', 'deactivate']
            );

            updateCollectIdFlag(
                $childrenUL,
                'dont-collect'
            );
        } else if (state !== 'active') {
            updateCheckboxStatesList(
                $childrenUL.closest('ul.children').removeClass('bf-checkbox-dual-state'),
                ['none', 'active', 'deactivate']
            );

            updateCollectIdFlag(
                $childrenUL,
                'collect'
            );
        }

        var canCreateLink = canCreatePrimaryLink($container);

        toggleShowPrimaryClass($container, canCreateLink);

        $('.bf-checkbox-status', $container).each(function () {
            var termID = getTermIdByInputName(this.name),
                $this = $(this),
                status = 'none',
                $container = $this.closest('.bf-checkbox-multi-state'),
                isTermRoot = $this.closest('ul.children', $container).length === 0,
                $label = $container.next('.label');

            if (canCollectTermAsDeactivated(this)) {
                termsIdList.push("-" + termID);
                status = 'deactivate';
            } else if (canCollectTermAsActivated(this, $container)) {

                status = 'active';
                if (isTermPrimary($label)) {
                    termsIdList.unshift("+" + termID);
                } else {
                    termsIdList.push(termID);
                }
            }
            $label.attr('data-status', status)
                .data('status', status);

            /**
             * Append Make Primary label if needed
             */
            if (canCreateLink && isTermRoot && !$container.hasClass('bf-checkbox-primary-term')) {
                if (this.value === 'active') {
                    createPrimaryLink($label, termID);
                } else {
                    //$label.find(".bf-make-term-primary").remove();
                }
            }

            /**
             * Append Excluded label if needed
             */
            if (this.value === 'deactivate') {
                if (!$label.find('.bf-excluded-term').length) {

                    var $el = $('<em/>', {
                        'class': 'bf-excluded-term'
                    }).text(__('Excluded', 'better-studio'));

                    $el.appendTo($label);
                }
            } else {
                $label.find('.bf-excluded-term').remove();
            }
        }).promise().done(function () {

            var newValue = termsIdList.join(',');
            var $input = $container.nextAll('.bf-term-select-value');

            if ($input.length) {

                $input.val(newValue);
                $input.change()[0].dispatchEvent(new Event('change', {bubbles: true}));
            }

            onChange(newValue);
        });
    }, 40));

    /**
     * Handle Make Primary Action
     */
    $context.on('click.term_select', '.bf-make-term-primary a', function (e) {

        e.preventDefault();

        var $this = $(this),
            termID = $this.data('term-id'),
            $container = $this.closest('.bf-field-term-select-wrapper'),
            $inputEL = $container.siblings('.bf-term-select-value');

        /**
         * Regenerate terms id and mark primary term with a + sign
         */
        var termsIdList = ["+" + termID],
            canCreateLink = canCreatePrimaryLink($container);

        toggleShowPrimaryClass($container, canCreateLink);

        $('.bf-checkbox-status', $container).each(function () {
            var _termID = getTermIdByInputName(this.name);

            if (_termID == termID) {
                return;
            }

            if (canCollectTermAsDeactivated(this)) {
                termsIdList.push("-" + _termID);
            } else if (canCollectTermAsActivated(this)) {
                termsIdList.push(_termID);

            }

        }).promise().done(function () {
            var newValue = termsIdList.join(',');
            $inputEL.val(newValue).change();

            onChange(newValue);
        });

        if (canCreateLink) {
            $('.bf-is-term-primary', $container).each(function () {
                createPrimaryLink($(this).parent());
            });
        }

        var $label = $this.parent();

        $label.addClass('bf-is-term-primary')
            .html('Primary');

        /**
         * Mark term as primary
         */
        $('.bf-checkbox-multi-state', $container).removeClass('bf-checkbox-primary-term');
        $label.closest('.label').prevAll('.bf-checkbox-multi-state').addClass('bf-checkbox-primary-term');
    });

    initCheckboxes($context);

    /**
     * Load Deferred Items.
     */

    const $input = $context.find(".bf-term-select-value");
    const $deferred = $(".bf-field-term-select-deferred", $context);

    UI.block($context);

    const taxonomy = $deferred.data('taxonomy');
    load_data("term_select", {taxonomy}, `term_select.${taxonomy}`)
        .then((response) => {

            try {
                UI.unblock($context);

                if (response.raw) {

                    const selected = $input.val();
                    $deferred.html('<ul>' + response.raw + '</ul>')
                        .removeClass('bf-field-term-select-deferred');

                    BetterStudio?.Libs?.checkboxMultiState($context, selected);
                    initCheckboxes($context);

                    makePrimary(findPrimaryTermId(selected));
                }
            } catch (e) {

                console.error(e);
            }

        }).catch((error) => {
        console.group("catch");
        console.error(error);
        console.groupEnd();
        UI.unblock($context);
        $deferred.html("ERROR fetch data.");
    });

    var $container = $('.bf-field-term-select-wrapper', $context);
    toggleShowPrimaryClass($container, canCreatePrimaryLink($container));

    scrollToFirstActiveCheckbox($context);

    return {
        $context,
        $input
    };
}
