// @flow
import {ControlBase, ControlInterface, ControlHaveData} from "../../ControlBase"

export default class extends ControlBase implements ControlInterface, ControlHaveData {

    context: HTMLElement;

    constructor(props: ?ControlProps) {

        super(props);

        this.onInputChanged = this.onInputChanged.bind(this);
    }

    controlType(): string {

        return 'text';
    }

    init(element: HTMLElement): boolean {

        this.context = element;

        this.bindEvents();

        return true;
    }

    input() {

        return this.context.querySelector("input[type='text']");
    }

    bindEvents() {

        const input = this.input();

        input && input.addEventListener('change', this.onInputChanged)
    }

    onInputChanged(event) {

        this.valueSet(event.target.value);
    }

    valueSet(value: mixed): boolean {

        this.onChange(value);

        return true;
    }

    valueGet(): any {

        const input = this.input();

        return input ? input.value : "";
    }

    destroy(): void {

        const input = this.input();

        input && input.removeEventListener('change', this.onInputChanged)
    }

    dataType(): string {

        return 'string';
    }
}