// @flow
import {ControlBase, ControlInterface, ControlHaveData} from "../../ControlBase"

export default class extends ControlBase implements ControlInterface, ControlHaveData {

    context: HTMLElement;

    constructor(props: ?ControlProps) {

        super(props);

        this.onTextareaChanged = this.onTextareaChanged.bind(this);
    }

    controlType(): string {

        return 'textarea';
    }

    init(element: HTMLElement): boolean {

        this.context = element;

        this.bindEvents();

        return true;
    }

    textarea(): ?HTMLTextAreaElement {

        return this.context.querySelector("textarea");
    }

    bindEvents() {

        const element = this.textarea();

        element && element.addEventListener('change', this.onTextareaChanged)
    }

    onTextareaChanged(event) {

        this.valueSet(event.target.value);
    }

    valueSet(value: mixed): boolean {

        this.onChange(value);

        return true;
    }

    valueGet(): any {

        const element = this.textarea();

        return element ? element.value : "";
    }

    destroy(): void {

        const input = this.textarea();

        input && input.removeEventListener('change', this.onTextareaChanged)
    }

    dataType(): string {

        return 'string';
    }
}