<?php

namespace BetterStudio\Component\Control\Typography;

// use current component
use BetterStudio\Component\Control as LibRoot;

// use standard APIs
use BetterStudio\Component\Standard\{
	Control as ControlStandard
};

class TypographyControl extends LibRoot\BaseDataControl implements
	ControlStandard\HaveScripts,
	ControlStandard\WillModifyProps,
	ControlStandard\HaveAjaxHandler {

	/**
	 * @since 1.0.0
	 * @return string
	 */
	public function control_type(): string {

		return 'typography';
	}

	/**
	 * @since 1.0.0
	 * @return string
	 */
	public function template_dir(): string {

		return __DIR__ . '/templates';
	}

	public function ajax_handler(): ControlStandard\HandleAjaxRequest {

		return new TypographyAjaxHandler();
	}

	/**
	 * @inheritDoc
	 */
	public function scripts_list(): array {

		$this->load_media_assets();

		return [];
	}

	public function modify_props( array $props ): array {

		if ( ! empty( $props['parent_typo_options']['family'] ) ) {

			$props['family_input_value'] = $props['value']['family'];
			$props['value']['family']    = $props['parent_typo_options']['family'];
		}

		// Get current font
		$props['font'] = Helpers::font( $props['value']['family'] ?? '' );

		if ( ! $props['font'] ) {

			$props['font'] = [
				'type' => ''
			];
		}

		$props['variants_options'] = Helpers::font_variants_option_elements( $props['font'], $props['value']['variant'] ?? '' );
		$props['subset_options']   = Helpers::font_subset_option_elements( $props['font'], $props['value']['subset'] ?? '' );

		if ( ! isset( $props['classes'] ) ) {
			$props['classes'] = [];
		}

		$props['classes'] = (array) $props['classes'];

		if ( ! isset( $props['container_attributes'] ) ) {
			$props['container_attributes'] = [];
		}

		$is_top_level_parent_font = isset( $props['parent_typo'] ) && $props['parent_typo'] === true;

		if ( $is_top_level_parent_font ) {

			$props['container_attributes']['data-control-id'] = $props['id'];
			$props['classes'][]                               = 'bf-parent-font-section';

		} elseif ( ! empty( $props['parent_typo'] ) ) {

			$props['classes'][] = 'bf-child-font-section';
			//
			$props['container_attributes']['data-parent-font'] = $props['parent_typo'];
		}

		return $props;
	}

	public function data_type(): string {

		return 'object';
	}
}
