const path = require("path");
const UglifyJsPlugin = require('terser-webpack-plugin');
const DependencyExtractionWebpackPlugin = require('@wordpress/dependency-extraction-webpack-plugin');

// Set default entry before loading wp webpack.config
process.env.WP_ENTRY = JSON.stringify({index: path.join(__dirname, "src/gutenberg.js")});

module.exports = (env) => {

    const isDev = !env.WEBPACK_BUILD && !env.WEBPACK_BUNDLE;

    const config = {
        entry: path.resolve(__dirname, "src/gutenberg.js"),
        output: {
            path: path.resolve(__dirname, "dist"),
            filename: "controls-gutenberg.js",
            library: ["BetterStudio", "Block"],
            libraryTarget: "var",
        },
        module: {
            rules: [
                {
                    test: /\.(js)$/,
                    exclude: /node_modules/,
                    use: "babel-loader",
                }
            ],
        },
        optimization: {
            minimize: !isDev,
            minimizer: [new UglifyJsPlugin()],
        },
        mode: isDev ? "development" : "production",
        plugins: [new DependencyExtractionWebpackPlugin()],
        target: ['web', 'es5'],
    };

    if (isDev) {

        config.devtool = "source-map";
    }

    return config;
}