const path = require("path");
const UglifyJsPlugin = require('terser-webpack-plugin');
const DependencyExtractionWebpackPlugin = require("@betterstudio/dependency-extraction-webpack-plugin");

module.exports = (env, options) => {

    const isDev = options.mode === "development";

    const config = {
        entry: path.resolve(__dirname, "src/controls.js"),
        output: {
            path: path.resolve(__dirname, "dist"),
            filename: "controls-script.js",
            library: ["BetterStudio", "Controls"],
            libraryTarget: "var",
        },
        module: {
            rules: [
                {
                    test: /\.(js)$/,
                    exclude: /node_modules/,
                    use: "babel-loader",
                }
            ],
        },
        optimization: {
            minimize: !isDev,
            minimizer: [new UglifyJsPlugin()],
        },
        mode: isDev ? "development" : "production",
        plugins: [new DependencyExtractionWebpackPlugin({
            requiredScripts: [
                "wp-api-request",
                "wp-color-picker",
            ]
        })],
        target: ['web', 'es5'],
    }

    if (isDev) {

        config.devtool = "source-map";
    }

    return config;
}
