class CssUtils {

    static disable(target) {

        target.style.pointerEvents = 'none';
        target.style.userSelect = 'none';
        target.style.opacity = 0.5;
        target.classList.add('show-on-disabled');
    }

    static enable(target) {

        target.style.removeProperty('pointer-events');
        target.style.removeProperty('user-select');
        target.style.removeProperty('opacity');
        target.classList.remove('show-on-disabled');
    }
}


class ElementUtils {

    static closest(element,selector,limit) {

        for(var i =0; i<limit;i++){

            element = element.parentElement;

            if(! element) {

                return ;
            }

            if(element.matches(selector)) {

                return element;
            }
        }
    }
}

class SlideElement {

    static clear(element) {

        [
            'height', 'padding-top', 'padding-bottom', 'transition-property',
            'margin-top', 'margin-bottom', 'transition-duration', 'overflow',
        ].forEach((cssProp) => element.style.removeProperty(cssProp));

        element.dataset.timeoutID && clearInterval(element.dataset.timeoutID);

        delete element.dataset.timeoutID;
    }

    static slideUp(target, duration = 400) {

        SlideElement.clear(target);

        if (window.getComputedStyle(target).display === 'none') {

            return;
        }

        target.style.transitionProperty = "height, margin, padding";
        target.style.overflow = 'hidden';
        target.style.transitionDuration = duration + 'ms';
        target.style.boxSizing = 'border-box';
        target.style.height = target.offsetHeight + 'px';
        target.offsetHeight;
        target.style.height = 0;
        target.style.paddingTop = 0;
        target.style.paddingBottom = 0;
        target.style.marginTop = 0;
        target.style.marginBottom = 0;

        target.dataset.timeoutID = window.setTimeout(() => {

            SlideElement.hide(target);
            SlideElement.clear(target);
        }, duration);
    }

    static slideDown(target, duration = 400) {

        SlideElement.clear(target);

        if (window.getComputedStyle(target).display !== 'none') {

            return;
        }

        SlideElement.show(target);

        let display = window.getComputedStyle(target).display;
        if (display === 'none') display = 'block';
        target.style.display = display;
        const height = target.offsetHeight;
        target.style.transitionProperty = "height, margin, padding";
        target.style.overflow = 'hidden';
        target.style.height = 0;
        target.style.paddingTop = 0;
        target.style.paddingBottom = 0;
        target.style.marginTop = 0;
        target.style.marginBottom = 0;
        target.offsetHeight;
        target.style.boxSizing = 'border-box';
        target.style.transitionDuration = duration + 'ms';
        target.style.height = height + 'px';
        target.style.removeProperty('padding-top');
        target.style.removeProperty('padding-bottom');
        target.style.removeProperty('margin-top');
        target.style.removeProperty('margin-bottom');
        target.dataset.timeoutID = window.setTimeout(() => {

            SlideElement.clear(target);
        }, duration);
    }

    static slideToggle(target, duration = 400) {

        if (window.getComputedStyle(target).display === 'none') {

            SlideElement.slideDown(target, duration);

        } else {

            SlideElement.slideUp(target, duration);
        }
    }

    static show(target) {

        target.style.removeProperty('display');
    }

    static hide(target) {

        target.style.display = 'none';
    }
}

class Util {

    static sanitizeAttr(string) {

        return string.toString().replace(/([A-Z])/g, function (char) {
            return '-' + char.toLocaleLowerCase()
        });
    }


    static inputSelectors() {

        return [
            "input:checked",
            "input,select,textarea,button"
        ];
    }

    static findInput(wrapperEl) {

        if (!wrapperEl instanceof HTMLElement) {

            return;
        }

        if (wrapperEl.tagName.match(/^input|select|textarea$/i)) {

            return wrapperEl;
        }

        for (const selector of Util.inputSelectors()) {

            const input = wrapperEl.querySelector(selector);

            if (input) {

                return input;
            }
        }
    }
}

class ShowOnModule {

    constructor({onRefresh}) {

        this.rulesSet = {};
        this.current = {fieldId: null, fieldEl: null, wrapperEl: null};
        this.valuesStore = {};
        this.onRefresh = onRefresh || function () {
        };
    }

    iterate(iterationCallback, doneCallback, options = {}) {

        throw "Override iterate method";
    }


    value(fieldId) {

        const value = this.getStaticValue(fieldId);

        if (typeof value !== "undefined") {

            return value;
        }

        /// TODO: check if it's implement the interface

        return this.dynamicValue(fieldId, this.current);
    }

    dynamicValue(fieldId, state) {

        throw "Override value method";
    }

    /**
     * @param {boolean}  withAnimation
     */
    refresh(withAnimation = true) {


        this.iterate((fieldId, fieldEl, wrapperEl) => {

            this.current = {fieldId, fieldEl, wrapperEl};

            const conditions = this.rulesSet[fieldId];
            const options = this.rulesSetOptions[fieldId] || {};
            //
            const [evaluation, matchedRuleIndex] = this.evaluate(conditions);

            const show_on_type = options.show_on_type && options.show_on_type[matchedRuleIndex] || "hidden";
            options.animationDuration = show_on_type === "disable" ? 0 : (options.animationDuration || 400);
            options.matchedRuleIndex = matchedRuleIndex;

            this.onRefresh(wrapperEl, evaluation === "matched", options);

            switch (evaluation) {

                case "matched":

                    if (show_on_type === "disable") {

                        CssUtils.enable(wrapperEl);

                    } else if (show_on_type === "hide") {

                        SlideElement.show(wrapperEl);

                    } else {

                        SlideElement[withAnimation ? 'slideDown' : 'show'](wrapperEl, options.animationDuration);
                    }

                    break;

                case "not-matched":

                    if (show_on_type === "disable") {

                        CssUtils.disable(wrapperEl);

                    } else if (show_on_type === "hide") {

                        SlideElement.hide(wrapperEl);

                    } else {

                        SlideElement[withAnimation ? 'slideUp' : 'hide'](wrapperEl, options.animationDuration);
                    }

                    break;
            }
        });
    }

    rules(rules) {

        if (typeof rules === 'undefined') {

            return this.rulesSet;

        } else {

            this.rulesSet = rules;
        }
    }

    rulesOptions(options) {

        if (typeof options === 'undefined') {

            return this.rulesSetOptions;

        } else {

            this.rulesSetOptions = options;
        }
    }

    staticValues(values) {

        if (typeof values === 'undefined') {

            return this.valuesStore;

        } else {

            Object.assign(this.valuesStore, values);
        }
    }

    setStaticValue(id, value) {

        this.valuesStore[id] = value;
    }

    getStaticValue(id) {

        return this.valuesStore[id];
    }

    /**
     * @param {Array} conditions
     *
     * @return {Number}
     *
     * 1 => show
     * 2 => hide
     * 0 => unknown
     */
    evaluate(conditions) {

        if (!conditions || !Array.isArray(conditions)) {
            return ['unknown', 0];
        }

        /**
         * fix for 1-dimension show-on
         */
        if (!Array.isArray(conditions[0])) {

            conditions = [conditions];
        }

        for (let i = 0; i < conditions.length; i++) {

            const statementResult = this.evaluateSubConditions(
                typeof Array.isArray(conditions[i]) ? conditions[i] : [conditions[i]]
            );

            if (statementResult) {

                return ['matched', i];
            }
        }

        return ['not-matched', 0];
    }

    evaluateSubConditions(subConditions) {

        let result = false,
            value;

        for (let i = 0; i < subConditions.length; i++) {

            const split = subConditions[i].match(/(.*?)(\!?\=)(.*)/);

            if (!split) {
                continue;
            }

            const [, _param_name, operator, _param_value] = split;

            value = this.value(_param_name);

            if (value !== 'undefined') {

                switch (operator) {
                    case '=':
                        result = value === _param_value;
                        break;
                    case '!=':
                        result = value !== _param_value;
                        break;
                }
            }

            if (!result) {
                break;
            }
        }

        return result;
    }
}

export class showOn extends ShowOnModule {

    constructor(wrapperEl, options = {}) {

        super(options);

        const defaults = {
            cacheValues: true,
            bindChangeEvent: true,
            groupSelector: '.bf-group-inner,.group,form',
            showOnLocation: {
                selector: ".bf-section-container,.fields-group",
                optionsDataset: "paramSettings",
                nameDataset: "paramName",
            },
            repeaterItemSelector: '.bf-repeater-item',
            dynamicInputWrapper: '.bf-repeater-item',

            isRepeaterControl: (inputEl) => {

                if (!inputEl) {

                    return false;
                }

                return inputEl.classList.contains('bf-repeater') ||
                    inputEl.classList.contains('bf-repeater-controls-option') ||
                    !! ElementUtils.closest(inputEl,'.repeater-item-container,.bf-repeater-item', 9);
            }
        }

        this.options = Object.assign({}, defaults, options || {});

        this.context = wrapperEl;

        if (!this.isInitialized()) {

            this.bindEvents();
        }

        this.setup(wrapperEl);
    }

    wrapper() {

        return this.context;
    }


    setup(wrapperEl) {

        this.shortInit(wrapperEl);

        const {rules, values, options} = this.generate();

        this.staticValues(values)
        this.rules(rules);
        this.rulesOptions(options);
    }

    shortInit(wrapperEl, withAnimation = false) {

        this.iteratorWrapper = wrapperEl;

        setTimeout(() => this.refresh(withAnimation));
    }

    /**
     * @since 1.0.0
     * @returns {boolean}
     */
    isInitialized() {

        return this.context.dataset.showOnInitialized || false;
    }


    /**
     * Bind Change Events
     */
    bindEvents() {

        if (this.options.bindChangeEvent) {

            this.context.addEventListener('change', this.onChange.bind(this), false);
        }
    }

    onChange(event) {

        if (this.options.isRepeaterControl(event.target)) { // FIX: on repeater field, limit changes to active repeater item .

            this.iteratorWrapper = event.target.closest(this.options.repeaterItemSelector);

        } else {

            this.iteratorWrapper = event.target.closest(this.options.groupSelector);

            this.setStaticValue(
                this.fieldId(event.target),
                event.target.value
            );
        }

        this.refresh(true);
    }

    modifySelector(selector, append) {

        return selector.split(',').map(function (selector) {
            return selector + append;
        }).join(',');
    }

    /**
     * @param {function} callback
     * @param {function} done
     * @param {object} options
     *
     * @since 1.0.0
     */
    iterate(callback, done, options = {}) {

        const {selector, optionsDataset} = this.options.showOnLocation;

        ///// duplicated
        const iterateSelector = options?.all || !optionsDataset ? selector : this.modifySelector(selector, `[data-${Util.sanitizeAttr(optionsDataset)}]`);

        this.iteratorWrapper
            .querySelectorAll(iterateSelector)
            .forEach((inputWrapper) => {

                const input = Util.findInput(inputWrapper),
                    id = this.fieldId(input, inputWrapper);

                callback(id, input, inputWrapper);
            });

        done && done();
    }

    fieldId(field, fieldWrapper = null) {

        const {selector, nameDataset} = this.options.showOnLocation;

        const findDataSet = (element) => {

            if (!element || !element.dataset) {

                return null;
            }

            if (element.dataset[nameDataset]) {


                return element.dataset[nameDataset];
            }

            if (element.dataset.showOnId) {

                return element.dataset.showOnId;
            }
        };

        let theID;

        theID = findDataSet(field);

        if (theID) {

            return theID;
        }

        if (!fieldWrapper && field) {
            fieldWrapper = field.closest(selector);
        }

        theID = findDataSet(fieldWrapper);

        if (theID) {

            return theID;
        }

        if (nameDataset) {

            const input = fieldWrapper.querySelector(`[data-${Util.sanitizeAttr(nameDataset)}], [data-show-on-id]`);

            return input && findDataSet(input);
        }

        return null;
    }

    findField(fieldId, fieldWrapper) {

        const {nameDataset} = this.options.showOnLocation;

        /// duplicated

        const uniqueWrapperSelector = `[data-${Util.sanitizeAttr(nameDataset)}="${fieldId}"]`,
            uniqueWrapper = fieldWrapper.querySelector(uniqueWrapperSelector);

        return Util.findInput(uniqueWrapper);
    }

    generate() {

        const {optionsDataset} = this.options.showOnLocation;
        const rules = {},
            options = {},
            values = {};

        this.iterate((fieldId, fieldEl, wrapperEl) => {

            const showOnString = optionsDataset ? wrapperEl.dataset[optionsDataset] : false;

            if (showOnString) {

                const {show_on, show_on_type} = JSON.parse(showOnString);

                rules[fieldId] = show_on;
                options[fieldId] = {show_on_type};
            }

            /// the repeater fields value should check dynamically
            if (this.options.cacheValues && fieldEl && !this.options.isRepeaterControl(fieldEl, wrapperEl)) {

                values[fieldId] = fieldEl.value;
            }

        }, null, {all: true});

        return {rules, options, values};
    }

    dynamicValue(fieldId, {wrapperEl}) {

        const rItemEl = wrapperEl.closest(this.options.dynamicInputWrapper);

        if (!rItemEl) {

            return;
        }

        const field = this.findField(fieldId, rItemEl);

        return field && field.value;
    }
}