const path = require("path");
const UglifyJsPlugin = require('terser-webpack-plugin');
const DependencyExtractionWebpackPlugin = require('@wordpress/dependency-extraction-webpack-plugin');

module.exports = (env, options) => {

    const isDev = options.mode === "development";

    const config = {
        entry: path.resolve(__dirname, "src/show-on.js"),
        output: {
            path: path.resolve(__dirname, "dist"),
            filename: "show-on.js",
            library: ["BetterStudio", "Libs"],
            libraryTarget: "assign-properties",
            iife: true,
        },
        module: {
            rules: [
                {
                    test: /\.(js)$/,
                    exclude: /node_modules/,
                    use: "babel-loader",
                }
            ],
        },
        optimization: {
            minimize: !isDev,
            minimizer: [new UglifyJsPlugin()],
        },
        mode: isDev ? "development" : "production",
        plugins: [new DependencyExtractionWebpackPlugin()],
        target: ['web', 'es5'],
    };

    if (isDev) {

        config.devtool = "source-map";
    }

    return config;
}
