<?php

namespace BetterStudio\Component\Control\Core\Validation;

// use utilities
use BetterStudio\Utils\{
	Validator
};

// use standard APIs
use BetterStudio\Component\Standard\{
	Control as ControlStandard
};

// use integration APIs
use BetterStudio\Component\Integration\{
	Control as ControlIntegration
};

class ControlValidation extends Validator\Rules\Rule {

	/**
	 * @inheritDoc
	 */
	public function check( $value ): bool {

		if ( ! $instance = ControlIntegration\ControlsStorage::factory(
			$value
		) ) {

			return false;
		}

		if ( $this->options['handle_ajax_request'] && ! $instance instanceof ControlStandard\HaveAjaxHandler ) {

			return false;
		}

		if ( $this->options['modify_settings'] && ! $instance instanceof ControlStandard\HaveSecureProps ) {

			return false;
		}

		return true;
	}

	/**
	 * @inheritDoc
	 */
	public function default_options(): array {

		return [
			'handle_ajax_request' => false,
			'modify_settings'     => false,
		];
	}

	/**
	 * @inheritDoc
	 */
	public function unnamed_option(): string {

		return '';
	}


	/**
	 * @inheritDoc
	 */
	protected function error_message( $placeholders ): string {

		return __( 'Invalid control.', 'better-studio' );
	}
}
