<?php

namespace BetterStudio\Component\Control\Select;

use Walker_CategoryDropdown;

if ( ! class_exists( Walker_CategoryDropdown::class ) ) {

	require ABSPATH . WPINC . '/class-walker-category-dropdown.php';
}

class DropdownWalker extends \Walker_CategoryDropdown {

	/**
	 * Start the element output.
	 *
	 * @see   Walker::start_el()
	 * @since 1.0.0
	 *
	 * @param string $output   Passed by reference. Used to append additional content.
	 * @param object $category Category data object.
	 * @param int    $depth    Depth of category. Used for padding.
	 * @param array  $args     Uses 'selected', 'show_count', and 'value_field' keys, if they exist.
	 *                         See {@see wp_dropdown_categories()}.
	 */
	public function start_el( &$output, $category, $depth = 0, $args = [], $id = 0 ) {

		$pad = \str_repeat( '&nbsp;', $depth * 3 );

		/** This filter is documented in wp-includes/category-template.php */
		$cat_name = \apply_filters( 'list_cats', $category->name, $category );

		if ( isset( $args['value_field'], $category->{$args['value_field']} ) ) {
			$value_field = $args['value_field'];
		} else {
			$value_field = 'term_id';
		}

		$output .= "\t<option class=\"level-$depth\" value=\"" . esc_attr( $category->{$value_field} ) . "\"";

		//
		// Changed to support multiple
		//
		if ( \is_array( $args['selected'] ) ) {
			if ( \in_array( $category->{$value_field}, $args['selected'], false ) ) {
				$output .= ' selected="selected"';
			}
		} elseif ( (string) $category->{$value_field} === (string) $args['selected'] ) {
			$output .= ' selected="selected"';
		}

		$output .= '>';
		$output .= $pad . $cat_name;
		if ( $args['show_count'] ) {
			$output .= '&nbsp;&nbsp;(' . number_format_i18n( $category->count ) . ')';
		}
		$output .= "</option>\n";
	}
}
